@@ -1,59 +1,46 @@
-#define VALBITS         24
-#define VALMASK         ((1 << VALBITS) - 1)
-#define SIGNBIT         (1 << (VALBITS-1))
-#define SIGNMASK        ~(SIGNBIT-1)
-#define FIXNUM_FITS(x)  (((x) & SIGNMASK) == 0 || ((x) & SIGNMASK) == SIGNMASK)
-#define FIXNUM_FITS_UNSIGNED(x)     (((x) & SIGNMASK) == 0)
-
-#define CONSTBIT        (1 << VALBITS)
-
-#define TYPESHIFT       (VALBITS+1)
-
-typedef unsigned Object;
-
-#define MAX_TYPE        (1 << (sizeof(Object)*8 - TYPESHIFT))
-
-/* Extract/Set/Compare the type and val components of Objects:
+/* The Scheme object representation, and a few other important
+ * data types.
  */
 
-#define TYPE(x) ((int)((x) >> TYPESHIFT))
-#define SETTYPE(x,t) ((x) = ((x) & VALMASK) | ((x) & CONSTBIT) | \
-			    ((int)(t) << TYPESHIFT))
+typedef struct {
+    unsigned long data;
+    int tag;
+} Object;
 
-#if (-1 >> 1) < 0
-#  define FIXNUM(x) (((int)(x) << (32-VALBITS)) >> (32-VALBITS))
-#else
-#  define FIXNUM(x) ((int)((x) & SIGNBIT ? (x) | ~VALMASK : (x)))
-#endif
+#define FIXBITS         (8 * sizeof(int))
+#define SIGNBIT         ((unsigned)1 << (FIXBITS-1))
+#define CONSTBIT        1
+#define TYPEBITS        (8 * sizeof(int) - 1)
+#define MAX_TYPE        ((1 << TYPEBITS) - 1)
 
-#define SETFIXNUM(x,i) ((x) = ((x) & ~VALMASK) | ((i) & VALMASK))
+#define UFIXNUM_FITS(i) (((i) & SIGNBIT) == 0)
+#define FIXNUM_FITS(i)  1
 
-#define CHAR(x) ((int)((x) & VALMASK))
+#define TYPE(x)         ((x).tag >> 1)
 
-#ifndef POINTER_CONSTANT_HIGH_BITS
-#  define POINTER(x) ((x) & VALMASK)
-#else
-#  define POINTER(x) (((x) & VALMASK) | POINTER_CONSTANT_HIGH_BITS)
-#endif
-#define SETPOINTER(x,p) SETFIXNUM(x,(int)(p))
+#define FIXNUM(x)       ((int)(x).data)
+#define CHAR(x)         ((int)(x).data)
 
-#define ISCONST(x) ((x) & CONSTBIT)
-#define SETCONST(x) ((x) |= CONSTBIT)
+#define POINTER(x)      ((x).data)
+#define SETPOINTER(x,p) ((x).data = (unsigned long)(p))
+#define SET(x,t,p)      ((x).tag = (int)t << 1, (x).data = (unsigned long)(p))
 
-#define SET(x,t,p) ((x) = ((int)(t) << TYPESHIFT) | ((int)(p) & VALMASK))
+#define ISCONST(x)      ((x).tag & CONSTBIT)
+#define SETCONST(x)     ((x).tag |= CONSTBIT)
 
-#define EQ(x,y) ((x) == (y))
-
-#define SETFAST(x,y) ((x) = (y))
+#define EQ(x,y)         ((x).data == (y).data && (x).tag == (y).tag)
 
 /* GC related macros:
  */
 #define WAS_FORWARDED(obj) (TYPE(*(Object *)POINTER(obj)) == T_Broken_Heart)
 #define UPDATE_OBJ(obj)    SETPOINTER(obj, POINTER(*(Object *)POINTER(obj)))
+
 #ifdef GENERATIONAL_GC
 
-   typedef int  gcspace_t;           /* type used for space and type arrays */
-   typedef unsigned gcptr_t;         /* type used for pointers */
+   typedef int  gcspace_t;            /* type used for space and type arrays */
+   typedef unsigned long gcptr_t;     /* type used for pointers */
+   typedef unsigned long pageno_t;    /* type used for page numbers */
+   typedef unsigned long addrarith_t; /* type used for address arithmetic */
 
    extern gcspace_t *space;
    extern gcspace_t current_space;
@@ -62,17 +49,16 @@ typedef unsigned Object;
    C_LINKAGE_END
 
 #  ifdef ARRAY_BROKEN
-   extern unsigned pagebase;
+   extern pageno_t pagebase;
 #  else
-#  define pagebase 0
+#  define pagebase ((pageno_t)0)
 #  endif
 
 #  define PAGEBYTES        512
 #  define PAGE_TO_OBJ(p)   ((Object *) (((p) + pagebase) * PAGEBYTES))
-#  define OBJ_TO_PAGE(p)   ((((gcptr_t)POINTER((Object)(p))) / PAGEBYTES) \
-			    - pagebase)
+#  define OBJ_TO_PAGE(p)   ((((gcptr_t)POINTER(p)) / PAGEBYTES) - pagebase)
 #  define STABLE(x)        ((~space[(x)]) & 1)
-#  define MAKEOBJ(t,p)     (((int)(t) << TYPESHIFT) | ((int)(p) & VALMASK))
+#  define MAKEOBJ(o,t,p)   (SET(o, t, p))
 #  define IS_ALIVE(obj)    ((WAS_FORWARDED(obj)) || \
                             (STABLE(OBJ_TO_PAGE(obj))) || \
 			    (space[OBJ_TO_PAGE(obj)] == current_space))
@@ -89,30 +75,26 @@ typedef unsigned Object;
 #define T_Flonum          2
 #define T_Null            3      /* empty list */
 #define T_Boolean         4      /* #t (1) and #f (0) */
-                                 /* 5 is unused; was T_Void */
-#define T_Unbound         6      /* only used internally */
-#define T_Special         7      /* only used internally */
-#define T_Character       8
-#define T_Symbol          9
-#define T_Pair           10
-#define T_Environment    11      /* A pair */
-#define T_String         12
-#define T_Vector         13
-#define T_Primitive      14      /* Primitive procedure */
-#define T_Compound       15      /* Compound procedure */
-#define T_Control_Point  16
-#define T_Promise        17      /* Result of (delay expression) */
-#define T_Port           18
-#define T_End_Of_File    19
-#define T_Autoload       20
-#define T_Macro          21
-#define T_Broken_Heart   22      /* only used internally */
+#define T_Unbound         5      /* only used internally */
+#define T_Special         6      /* only used internally */
+#define T_Character       7
+#define T_Symbol          8
+#define T_Pair            9
+#define T_Environment    10      /* A pair */
+#define T_String         11
+#define T_Vector         12
+#define T_Primitive      13      /* Primitive procedure */
+#define T_Compound       14      /* Compound procedure */
+#define T_Control_Point  15
+#define T_Promise        16      /* Result of (delay expression) */
+#define T_Port           17
+#define T_End_Of_File    18
+#define T_Autoload       19
+#define T_Macro          20
+#define T_Broken_Heart   21      /* only used internally */
 #ifdef GENERATIONAL_GC
-#  define T_Align_8Byte  23      /* only used internally */
-#  define T_Freespace    24      /* only used internally */
-#  define T_Last         T_Freespace
-#else
-#  define T_Last T_Broken_Heart
+#  define T_Align_8Byte  22      /* only used internally */
+#  define T_Freespace    23      /* only used internally */
 #endif
 
 #define BIGNUM(x)   ((struct S_Bignum *)POINTER(x))
@@ -144,9 +126,9 @@ struct S_Flonum {
 };
 
 struct S_Symbol {
+    Object value;
     Object next;
     Object name;               /* A string */
-    Object value;
     Object plist;
 };
 
@@ -181,6 +163,7 @@ struct S_Compound {
     Object closure;     /* (lambda (args) form ...) */
     Object env;         /* Procedure's environment */
     int min_args, max_args;
+    int numforms;
     Object name;
 };
 
@@ -221,14 +204,14 @@ struct S_Control {
     Object gcsave;              /* vector */
     WIND *firstwind, *lastwind;
     int tailcall;
-    unsigned long intrlevel;
     unsigned delta;
 #ifdef GENERATIONAL_GC
     int reloc;
 #endif
     jmp_buf j;
     int size;
-    char stack[1];
+    unsigned long intrlevel;
+    char stack[1];    /* must be word aligned */
 };
 
 struct S_Promise {
@@ -309,11 +292,11 @@ typedef struct {
 
 typedef struct {
     char *name;
-    int flags;
+    int type;
 } SYMPREFIX;
 
-#define PR_EXTENSION     0x1   /* Elk extension init/finit functions */
-#define PR_CONSTRUCTOR   0x2   /* C++ static constructors/destructors */
+#define PR_EXTENSION     0   /* Elk extension initializers/finalizers */
+#define PR_CONSTRUCTOR   1   /* C++ static constructors/destructors */
 
 
 /* PFO, GENERIC, and MATCHFUN exist for backwards compatibility
@@ -332,7 +315,6 @@ typedef struct weak_node {
 
 /* flags */
 #define WK_LEADER 1
-#define WK_MARK   2
 
 
 typedef struct {
