@@ -7,8 +7,8 @@ static Object P_Hack_Procedure_Environment (p, e) Object p, e; {
     return p;
 }
 
-init_lib_hack () {
+elk_init_lib_hack () {
     Define_Primitive (P_Hack_Procedure_Environment,
 	"hack-procedure-environment!", 2, 2, EVAL);
-    P_Provide (Intern ("hack"));
+    P_Provide (Intern ("hack.o"));
 }
