@@ -12,6 +12,6 @@ static Object P_Getpass(prompt) Object prompt; {
     return Make_String(ret, strlen(ret));
 }
 
-init_unix_misc() {
+elk_init_unix_misc() {
     Def_Prim(P_Getpass,          "unix-getpass",                1, 1, EVAL);
 }
