@@ -113,7 +113,7 @@ static Object P_Handle_Events (argl) Object argl; {
 	func = Eval (Car (Cdr (clause)));
 	Check_Procedure (func);
 	clause = Car (clause);
-	if (EQ(clause, Intern ("else"))) {
+	if (EQ(clause, Sym_Else)) {
 	    for (i = 0; i < LASTEvent; i++)
 		if (Nullp (funcs[i])) funcs[i] = func;
 	} else {
@@ -191,13 +191,13 @@ Object Get_Event_Args (ep) XEvent *ep; {
 	a[2] = Make_Window (0, p->display, p->root);
 	a[3] = Make_Window (0, p->display, p->subwindow);
 	a[4] = Get_Time_Arg (p->time);
-	a[5] = Make_Fixnum (p->x);
-	a[6] = Make_Fixnum (p->y);
-	a[7] = Make_Fixnum (p->x_root);
-	a[8] = Make_Fixnum (p->y_root);
+	a[5] = Make_Integer (p->x);
+	a[6] = Make_Integer (p->y);
+	a[7] = Make_Integer (p->x_root);
+	a[8] = Make_Integer (p->y_root);
 	if (e == KeyPress || e == KeyRelease) {
 	    a[9] = Bits_To_Symbols ((unsigned long)p->state, 1, State_Syms);
-	    a[10] = Make_Fixnum (p->keycode);
+	    a[10] = Make_Integer (p->keycode);
 	    a[11] = p->same_screen ? True : False;
 	} else if (e == ButtonPress || e == ButtonRelease) {
 	    register XButtonEvent *q = (XButtonEvent *)ep;
@@ -233,28 +233,28 @@ Object Get_Event_Args (ep) XEvent *ep; {
     case Expose: {
 	register XExposeEvent *p = (XExposeEvent *)ep;
 	a[1] = Make_Window (0, p->display, p->window);
-	a[2] = Make_Fixnum (p->x);
-	a[3] = Make_Fixnum (p->y);
-	a[4] = Make_Fixnum (p->width);
-	a[5] = Make_Fixnum (p->height);
-	a[6] = Make_Fixnum (p->count);
+	a[2] = Make_Integer (p->x);
+	a[3] = Make_Integer (p->y);
+	a[4] = Make_Integer (p->width);
+	a[5] = Make_Integer (p->height);
+	a[6] = Make_Integer (p->count);
     } break;
     case GraphicsExpose: {
 	register XGraphicsExposeEvent *p = (XGraphicsExposeEvent *)ep;
 	a[1] = Make_Window (0, p->display, p->drawable);
-	a[2] = Make_Fixnum (p->x);
-	a[3] = Make_Fixnum (p->y);
-	a[4] = Make_Fixnum (p->width);
-	a[5] = Make_Fixnum (p->height);
-	a[6] = Make_Fixnum (p->count);
-	a[7] = Make_Fixnum (p->major_code);
-	a[8] = Make_Fixnum (p->minor_code);
+	a[2] = Make_Integer (p->x);
+	a[3] = Make_Integer (p->y);
+	a[4] = Make_Integer (p->width);
+	a[5] = Make_Integer (p->height);
+	a[6] = Make_Integer (p->count);
+	a[7] = Make_Integer (p->major_code);
+	a[8] = Make_Integer (p->minor_code);
     } break;
     case NoExpose: {
 	register XNoExposeEvent *p = (XNoExposeEvent *)ep;
 	a[1] = Make_Window (0, p->display, p->drawable);
-	a[2] = Make_Fixnum (p->major_code);
-	a[3] = Make_Fixnum (p->minor_code);
+	a[2] = Make_Integer (p->major_code);
+	a[3] = Make_Integer (p->minor_code);
     } break;
     case VisibilityNotify: {
 	register XVisibilityEvent *p = (XVisibilityEvent *)ep;
@@ -265,11 +265,11 @@ Object Get_Event_Args (ep) XEvent *ep; {
 	register XCreateWindowEvent *p = (XCreateWindowEvent *)ep;
 	a[1] = Make_Window (0, p->display, p->parent);
 	a[2] = Make_Window (0, p->display, p->window);
-	a[3] = Make_Fixnum (p->x);
-	a[4] = Make_Fixnum (p->y);
-	a[5] = Make_Fixnum (p->width);
-	a[6] = Make_Fixnum (p->height);
-	a[7] = Make_Fixnum (p->border_width);
+	a[3] = Make_Integer (p->x);
+	a[4] = Make_Integer (p->y);
+	a[5] = Make_Integer (p->width);
+	a[6] = Make_Integer (p->height);
+	a[7] = Make_Integer (p->border_width);
 	a[8] = p->override_redirect ? True : False;
     } break;
     case DestroyNotify: {
@@ -299,19 +299,19 @@ Object Get_Event_Args (ep) XEvent *ep; {
 	a[1] = Make_Window (0, p->display, p->event);
 	a[2] = Make_Window (0, p->display, p->window);
 	a[3] = Make_Window (0, p->display, p->parent);
-	a[4] = Make_Fixnum (p->x);
-	a[5] = Make_Fixnum (p->y);
+	a[4] = Make_Integer (p->x);
+	a[5] = Make_Integer (p->y);
 	a[6] = p->override_redirect ? True : False;
     } break;
     case ConfigureNotify: {
 	register XConfigureEvent *p = (XConfigureEvent *)ep;
 	a[1] = Make_Window (0, p->display, p->event);
 	a[2] = Make_Window (0, p->display, p->window);
-	a[3] = Make_Fixnum (p->x);
-	a[4] = Make_Fixnum (p->y);
-	a[5] = Make_Fixnum (p->width);
-	a[6] = Make_Fixnum (p->height);
-	a[7] = Make_Fixnum (p->border_width);
+	a[3] = Make_Integer (p->x);
+	a[4] = Make_Integer (p->y);
+	a[5] = Make_Integer (p->width);
+	a[6] = Make_Integer (p->height);
+	a[7] = Make_Integer (p->border_width);
 	a[8] = Make_Window (0, p->display, p->above);
 	a[9] = p->override_redirect ? True : False;
     } break;
@@ -319,11 +319,11 @@ Object Get_Event_Args (ep) XEvent *ep; {
 	register XConfigureRequestEvent *p = (XConfigureRequestEvent *)ep;
 	a[1] = Make_Window (0, p->display, p->parent);
 	a[2] = Make_Window (0, p->display, p->window);
-	a[3] = Make_Fixnum (p->x);
-	a[4] = Make_Fixnum (p->y);
-	a[5] = Make_Fixnum (p->width);
-	a[6] = Make_Fixnum (p->height);
-	a[7] = Make_Fixnum (p->border_width);
+	a[3] = Make_Integer (p->x);
+	a[4] = Make_Integer (p->y);
+	a[5] = Make_Integer (p->width);
+	a[6] = Make_Integer (p->height);
+	a[7] = Make_Integer (p->border_width);
 	a[8] = Make_Window (0, p->display, p->above);
 	a[9] = Bits_To_Symbols ((unsigned long)p->detail, 0, Stack_Mode_Syms);
 	a[10] = Make_Unsigned_Long (p->value_mask);
@@ -332,14 +332,14 @@ Object Get_Event_Args (ep) XEvent *ep; {
 	register XGravityEvent *p = (XGravityEvent *)ep;
 	a[1] = Make_Window (0, p->display, p->event);
 	a[2] = Make_Window (0, p->display, p->window);
-	a[3] = Make_Fixnum (p->x);
-	a[4] = Make_Fixnum (p->y);
+	a[3] = Make_Integer (p->x);
+	a[4] = Make_Integer (p->y);
     } break;
     case ResizeRequest: {
 	register XResizeRequestEvent *p = (XResizeRequestEvent *)ep;
 	a[1] = Make_Window (0, p->display, p->window);
-	a[2] = Make_Fixnum (p->width);
-	a[3] = Make_Fixnum (p->height);
+	a[2] = Make_Integer (p->width);
+	a[3] = Make_Integer (p->height);
     } break;
     case CirculateNotify: {
 	register XCirculateEvent *p = (XCirculateEvent *)ep;
@@ -403,7 +403,7 @@ Object Get_Event_Args (ep) XEvent *ep; {
 	case 16:
 	    a[3] = Make_Vector (10, Null);
 	    for (i = 0; i < 10; i++)
-		VECTOR(a[3])->data[i] = Make_Fixnum (p->data.s[i]);
+		VECTOR(a[3])->data[i] = Make_Integer (p->data.s[i]);
 	    break;
 	case 32:
 	    a[3] = Make_Vector (5, Null);
@@ -418,8 +418,8 @@ Object Get_Event_Args (ep) XEvent *ep; {
 	register XMappingEvent *p = (XMappingEvent *)ep;
 	a[1] = Make_Window (0, p->display, p->window);
 	a[2] = Bits_To_Symbols ((unsigned long)p->request, 0, Mapping_Syms);
-	a[3] = Make_Fixnum (p->first_keycode);
-	a[4] = Make_Fixnum (p->count);
+	a[3] = Make_Integer (p->first_keycode);
+	a[4] = Make_Integer (p->count);
     } break;
     }
     a[0] = Intern (Event_Table[e].name);
@@ -470,11 +470,11 @@ static Object P_Get_Motion_Events (w, from, to) Object w, from, to; {
     e = ret = Make_Vector (n, Null);
     GC_Link2 (ret, e);
     for (i = 0; i < n; i++) {
-	e = P_Make_List (Make_Fixnum (3), Null);
+	e = P_Make_List (Make_Integer (3), Null);
 	VECTOR(ret)->data[i] = e;
 	Car (e) = Get_Time_Arg (p[i].time); e = Cdr (e);
-	Car (e) = Make_Fixnum (p[i].x); e = Cdr (e);
-	Car (e) = Make_Fixnum (p[i].y);
+	Car (e) = Make_Integer (p[i].x); e = Cdr (e);
+	Car (e) = Make_Integer (p[i].y);
     }
     GC_Unlink;
     XFree ((char *)p);
@@ -497,11 +497,11 @@ static Object P_Event_Listen (d, wait_flag) Object d, wait_flag; {
     return Make_Integer (n);
 }
 
-init_xlib_event () {
+elk_init_xlib_event () {
     Object t;
     register i;
 
-    Argl = P_Make_List (Make_Fixnum (MAX_ARGS), Null);
+    Argl = P_Make_List (Make_Integer (MAX_ARGS), Null);
     Global_GC_Link (Argl);
     Argv = Make_Vector (MAX_ARGS, Null);
     Global_GC_Link (Argv);
