@@ -12,15 +12,26 @@ static Object P_Xlib_Release_5_Or_Laterp () {
 #endif
 }
 
-init_xlib_init () {
+static Object P_Xlib_Release_6_Or_Laterp () {
+#ifdef XLIB_RELEASE_6_OR_LATER
+    return True;
+#else
+    return False;
+#endif
+}
+
+elk_init_xlib_init () {
     Define_Primitive (P_Xlib_Release_4_Or_Laterp,
 			"xlib-release-4-or-later?",               0, 0, EVAL);
     Define_Primitive (P_Xlib_Release_5_Or_Laterp,
 			"xlib-release-5-or-later?",               0, 0, EVAL);
+    Define_Primitive (P_Xlib_Release_6_Or_Laterp,
+			"xlib-release-6-or-later?",               0, 0, EVAL);
     P_Provide (Intern ("xlib.o"));
 }
 
-#if defined(XLIB_RELEASE_5_OR_LATER) && defined(sun)
+#if defined(XLIB_RELEASE_5_OR_LATER) && (defined(sun) || defined(__sun__)) &&\
+    defined(__svr4__)
 /*
  * Stub interface to dynamic linker routines
  * that SunOS uses but didn't ship with 4.1.
