@@ -33,15 +33,6 @@ int Match_X_Obj (x, v) Object x; va_list v; {
     return 0;
 }
 
-Object P_Window_Unique_Id (w) Object w; {
-    register id;
-
-    Check_Type (w, T_Window);
-    id = Unique_Id (w);
-    return id >= 0 ? Make_Fixnum (id) : False;
-}
-
-init_xlib_objects () {
+elk_init_xlib_objects () {
     Define_Symbol (&Sym_None, "none");
-    Define_Primitive (P_Window_Unique_Id, "window-unique-id", 1, 1, EVAL);
 }
