@@ -25,7 +25,7 @@ unsigned long Get_Pixel (p) Object p; {
 }
 
 static Object P_Pixel_Value (p) Object p; {
-    return Make_Unsigned ((unsigned)Get_Pixel (p));
+    return Make_Unsigned_Long (Get_Pixel (p));
 }
 
 static Object P_Black_Pixel (d) Object d; {
@@ -40,7 +40,7 @@ static Object P_White_Pixel (d) Object d; {
 	DefaultScreen (DISPLAY(d)->dpy)));
 }
 
-init_xlib_pixel () {
+elk_init_xlib_pixel () {
     Generic_Define (Pixel, "pixel", "pixel?");
     Define_Primitive (P_Pixel_Value,   "pixel-value",    1, 1, EVAL);
     Define_Primitive (P_Black_Pixel,   "black-pixel",    1, 1, EVAL);
