@@ -8,7 +8,7 @@ static Identifier_Equal (x, y) Object x, y; {
     return p->type == q->type && p->val == q->val && !p->free && !q->free;
 }
 
-Generic_Print (Identifier, "#[identifier %u]", POINTER(x))
+Generic_Print (Identifier, "#[identifier %lu]", POINTER(x))
 
 Object Make_Id (type, val, num) XtPointer val; {
     Object i;
@@ -35,6 +35,6 @@ XtPointer Use_Id (x, type) Object x; {
     return IDENTIFIER(x)->val;
 }
 
-init_xt_identifier () {
+elk_init_xt_identifier () {
     Generic_Define (Identifier, "identifier", "identifier?");
 }
