@@ -12,6 +12,14 @@ static Object P_Xt_Release_5_Or_Laterp () {
 #endif
 }
 
+static Object P_Xt_Release_6_Or_Laterp () {
+#ifdef XT_RELEASE_6_OR_LATER
+    return True;
+#else
+    return False;
+#endif
+}
+
 extern WidgetClass vendorShellWidgetClass;
 
 /* The reference to vendorShellWidgetClass is required to make sure
@@ -23,7 +31,7 @@ extern WidgetClass vendorShellWidgetClass;
 static dummy (w) WidgetClass w; {
 }
 
-init_xt_init () {
+elk_init_xt_init () {
     extern WidgetClass vendorShellWidgetClass;
 
     dummy(vendorShellWidgetClass);
@@ -32,6 +40,8 @@ init_xt_init () {
 	0, 0, EVAL);
     Define_Primitive (P_Xt_Release_5_Or_Laterp, "xt-release-5-or-later?",
 	0, 0, EVAL);
+    Define_Primitive (P_Xt_Release_6_Or_Laterp, "xt-release-6-or-later?",
+	0, 0, EVAL);
     XtToolkitInitialize ();
     P_Provide (Intern ("xt.o"));
 }
