@@ -1,4 +1,4 @@
-#include "scheme.h"
+#include "kernel.h"
 
 Object V_Autoload_Notifyp;
 
@@ -28,9 +28,9 @@ Object Do_Autoload (sym, al) Object sym, al; {
     Object val, a[1];
     GC_Node;
 
-    if (Truep (Var_Get (V_Autoload_Notifyp))) {
+    if (Var_Is_True (V_Autoload_Notifyp)) {
 	a[0] = AUTOLOAD(al)->files;
-	Format (Standard_Output_Port, "[Autoloading ~s]~%", 18, 1, a);
+	Format (Standard_Output_Port, "[Autoloading ~a]~%", 18, 1, a);
     }
     GC_Link (sym);
     (void)General_Load (AUTOLOAD(al)->files, AUTOLOAD(al)->env);
