@@ -65,15 +65,23 @@ Object Make_Bignum (buf, neg, radix) const char *buf; {
 }
 
 Object Reduce_Bignum (x) Object x; {
-    register i;
-    register struct S_Bignum *p = BIGNUM(x);
+    unsigned ret = 0;
+    int i, shift = 0, size = BIGNUM(x)->usize;
+    int digits = sizeof(int)/2;
 
-    if (p->usize > 2 || (p->usize == 2 && p->data[1] >= 32768))
+    if (size > digits)
 	return x;
-    i = Bignum_To_Integer (x);
-    if (!FIXNUM_FITS(i))
-	return x;
-    return Make_Fixnum (i);
+    for (i = 0; i < digits && i < size; i++, shift += 16)
+	ret |= (unsigned)BIGNUM(x)->data[i] << shift;
+    if (Truep (BIGNUM(x)->minusp)) {
+	if (ret > (~(unsigned)0 >> 1) + 1)
+	    return x;
+	return Make_Integer (-ret);
+    } else {
+	if (ret > ~(unsigned)0 >> 1)
+	    return x;
+	return Make_Integer (ret);
+    }
 }
 
 Bignum_Mult_In_Place (x, n) register struct S_Bignum *x; {
@@ -256,6 +264,18 @@ err:
     }
 }
 
+unsigned Bignum_To_Unsigned (x) Object x; {
+    unsigned ret = 0;
+    int i, shift = 0, size = BIGNUM(x)->usize;
+    int digits = sizeof(int)/2;
+
+    if (size > digits || Truep (BIGNUM(x)->minusp))
+	Primitive_Error ("integer out of range: ~s", x);
+    for (i = 0; i < digits && i < size; i++, shift += 16)
+	ret |= (unsigned)BIGNUM(x)->data[i] << shift;
+    return ret;
+}
+
 long Bignum_To_Long (x) Object x; {
     unsigned long ret = 0;
     int i, shift = 0, size = BIGNUM(x)->usize;
@@ -277,11 +297,24 @@ err:
     }
 }
 
+unsigned long Bignum_To_Unsigned_Long (x) Object x; {
+    unsigned long ret = 0;
+    int i, shift = 0, size = BIGNUM(x)->usize;
+    int digits = sizeof(long)/2;
+
+    if (size > digits || Truep (BIGNUM(x)->minusp))
+	Primitive_Error ("integer out of range: ~s", x);
+    for (i = 0; i < digits && i < size; i++, shift += 16)
+	ret |= (unsigned long)BIGNUM(x)->data[i] << shift;
+    return ret;
+}
+
 Object Integer_To_Bignum (i) {
     int k, digits = sizeof(int)/2;
-    Object big = Make_Uninitialized_Bignum (digits);
+    Object big; 
     unsigned n = i;
 
+    big = Make_Uninitialized_Bignum (digits);
     if (i < 0) {
 	BIGNUM(big)->minusp = True;
 	n = -i;
@@ -295,8 +328,9 @@ Object Integer_To_Bignum (i) {
 
 Object Unsigned_To_Bignum (i) unsigned i; {
     int k, digits = sizeof(int)/2;
-    Object big = Make_Uninitialized_Bignum (digits);
-
+    Object big;
+    
+    big = Make_Uninitialized_Bignum (digits);
     for (k = 0; k < digits; k++, i >>= 16)
 	BIGNUM(big)->data[k] = i & 0xffff;
     BIGNUM(big)->usize = k;
@@ -306,9 +340,10 @@ Object Unsigned_To_Bignum (i) unsigned i; {
 
 Object Long_To_Bignum (i) long i; {
     int k, digits = sizeof(long)/2;
-    Object big = Make_Uninitialized_Bignum (digits);
+    Object big;
     unsigned long n = i;
 
+    big = Make_Uninitialized_Bignum (digits);
     if (i < 0) {
 	BIGNUM(big)->minusp = True;
 	n = -i;
@@ -322,8 +357,9 @@ Object Long_To_Bignum (i) long i; {
 
 Object Unsigned_Long_To_Bignum (i) unsigned long i; {
     int k, digits = sizeof(long)/2;
-    Object big = Make_Uninitialized_Bignum (digits);
-
+    Object big;
+    
+    big = Make_Uninitialized_Bignum (digits);
     for (k = 0; k < digits; k++, i >>= 16)
 	BIGNUM(big)->data[k] = i & 0xffff;
     BIGNUM(big)->usize = k;
@@ -490,8 +526,9 @@ Object General_Bignum_Plus_Minus (x, y, neg) Object x, y; {
 	}
     } else {
 	if (Bignum_Mantissa_Cmp (BIGNUM(x), BIGNUM(y)) < 0) {
-	    Object temp = x;
-	    x = y; y = temp;
+	    Object temp;
+	    
+	    temp = x; x = y; y = temp;
 	    xsize = ysize;
 	    ysize = BIGNUM(y)->usize;
 	    xminusp = yminusp;
@@ -642,7 +679,7 @@ Object Bignum_Fixnum_Divide (x, y) Object x, y; {   /* bignum / fixnum */
     Bignum_Normalize_In_Place (BIGNUM(big));
     if (xminusp)
 	rem = -(int)rem;
-    return Cons (Reduce_Bignum (big), Make_Fixnum ((int)rem));
+    return Cons (Reduce_Bignum (big), Make_Integer ((int)rem));
 }
 
 /* Returns cons cell (quotient . remainder); cdr is a fixnum
@@ -657,7 +694,7 @@ Object Bignum_Divide (x, y) Object x, y; {   /* bignum / bignum */
     Alloca_Begin;
 
     if (BIGNUM(y)->usize < 2)
-	return Bignum_Fixnum_Divide (x, Make_Fixnum (Bignum_To_Integer (y)));
+	return Bignum_Fixnum_Divide (x, Make_Integer (Bignum_To_Integer (y)));
 
     GC_Link2 (x, y);
     quotsize = BIGNUM(x)->usize - BIGNUM(y)->usize + 1;
@@ -746,7 +783,8 @@ Object Bignum_Divide (x, y) Object x, y; {   /* bignum / bignum */
     if (Bignum_Div_In_Place (dend, scale))
 	Panic ("Bignum_Div scale");
  zero:
-    if (Truep (dend->minusp = BIGNUM(x)->minusp) != Truep (BIGNUM(y)->minusp))
+    dend->minusp = BIGNUM(x)->minusp;
+    if (Truep (dend->minusp) != Truep (BIGNUM(y)->minusp))
 	BIGNUM(quot)->minusp = True;
     Bignum_Normalize_In_Place (BIGNUM(quot));
     Bignum_Normalize_In_Place (dend);
