@@ -1,26 +1,24 @@
 /* Convert Scheme strings to C strings.  The contents of strings has to
- * be copied, because Elk's strings do not have a terminating null byte
- * (strings may *contain* null bytes).
+ * be copied, because strings in Elk do not have a terminating null-byte
+ * (strings may _contain_ null-bytes).
  *
- * Get_String() and Get_Strsym() allocate memory in NUMSTR cyclically
+ * Get_String() and Get_Strsym() allocate memory in NUMSTRBUFS cyclically
  * reused buffers in the C heap.
  * The macros Get_String_Stack() and Get_Strsym_Stack() in include/cstring.h
  * allocate memory on the stack.  They have to be used whenever more than
- * NUMSTR strings are active in a function at the same time.
+ * NUMSTRBUFS strings are active in a function at the same time.
  */
 
 #include "kernel.h"
 
-#define NUMSTR   3
-
-static char *heapstr[NUMSTR];
-static int heaplen[NUMSTR];
+static char *heapstr[NUMSTRBUFS];
+static int heaplen[NUMSTRBUFS];
 static int nextstr;
 
 Init_Cstring() {  /* Preallocate memory to avoid fragmentation */
     int i;
 
-    for (i = 0; i < NUMSTR; i++)
+    for (i = 0; i < NUMSTRBUFS; i++)
 	heapstr[i] = Safe_Malloc (heaplen[i] = 512);
 }
 
@@ -37,7 +35,7 @@ char *Get_String (str) Object str; {
     }
     bcopy (STRING(str)->data, *pp, --len);
     (*pp)[len] = '\0';
-    if (++nextstr == NUMSTR) nextstr = 0;
+    if (++nextstr == NUMSTRBUFS) nextstr = 0;
     return *pp;
 }
 
