@@ -1,4 +1,4 @@
-/* backtrace-list, etc.
+/* The primitive `backtrace-list'.
  */
 
 #include "kernel.h"
@@ -33,7 +33,7 @@ Object P_Backtrace_List (argc, argv) Object *argv; {
 	    if (Nullp (list))
 		list = cell;
 	    else
-		(void)P_Setcdr (tail, cell);
+		(void)P_Set_Cdr (tail, cell);
 	    tail = cell;
 	}
     }
