@@ -16,6 +16,7 @@ extern int errno;
 Object Dump_Control_Point;
 
 Init_Dump () {
+    Dump_Control_Point = Null;
     Global_GC_Link (Dump_Control_Point);
 }
 
@@ -31,6 +32,7 @@ Init_Dump () {
     char *ofn;\
     GC_Node;\
 \
+    Check_If_Dump_Works ();\
     if (!EQ (Curr_Input_Port, Standard_Input_Port) ||\
 	    !EQ (Curr_Output_Port, Standard_Output_Port))\
 	Primitive_Error ("cannot dump with current ports redirected");\
