@@ -1,6 +1,14 @@
 /* The generational, incremental garbage collector.
+ * Written by Marco Scheibe.  Fixes provided by Craig McPheeters,
+ * Carsten Bormann, Jon Hartlaub, Charlie Xiaoli Huang, Gal Shalif.
+ *
+ * This garbage collector is still experimental and probably needs to be
+ * rewritten at least in parts.  See also ../BUGS.  If your application
+ * does not work correctly and you suspect the generational garbage
+ * collector to be the culprit, try the stop-and-copy GC instead.
  */
 
+#include <limits.h>
 #include <sys/types.h>
 #ifdef HAS_MPROTECT
 #  include <sys/mman.h>
@@ -9,7 +17,7 @@
 #  define link FOO
 #  include <unistd.h>
 #  undef link
-#  ifdef _SC_PAGE_SIZE   /* It has the wrong name in HP-UX */
+#  if defined(_SC_PAGE_SIZE) && !defined(_SC_PAGESIZE)   /* Wrong in HP-UX */
 #    define _SC_PAGESIZE _SC_PAGE_SIZE
 #  endif
 #endif
@@ -46,46 +54,46 @@ typedef unsigned gcptr_t;	// type used for pointers
    ------------------------------------------------------------------------ */
 
 static int percent = 0;
-static int old_logical_pages;
+static pageno_t old_logical_pages;
 
 static int inc_collection = 0;
 
 static int incomplete_msg = 0;
 
-static int logical_pages, spanning_pages, physical_pages;
+static pageno_t logical_pages, spanning_pages, physical_pages;
 
 /* pagebase is #defined in object.h if ARRAY_BROKEN is not defined. */
 
 #ifdef ARRAY_BROKEN
-  unsigned pagebase;
+  pageno_t pagebase;
 #endif
 
-static unsigned firstpage, lastpage;
+static pageno_t firstpage, lastpage;
 
 gcspace_t *space;
 static gcspace_t *type, *pmap;
-static unsigned *link;
+static pageno_t *link;
 
-static unsigned current_pages, forwarded_pages;
-static unsigned protected_pages, allocated_pages;
+static pageno_t current_pages, forwarded_pages;
+static pageno_t protected_pages, allocated_pages;
 
-static unsigned bytes_per_pp, pp_shift;  /* bytes per physical page */
-static unsigned hp_per_pp;        /* number of heap pages per physical page */
-static unsigned pp_mask;          /* ANDed with a virtual address gives
-				   * base address of physical page
-				   */
-static unsigned hp_per_pp_mask;   /* ANDed with heap page number gives
-				   * first page number in the physical
-				   * page the heap page belongs to.
-				   */
+static addrarith_t bytes_per_pp, pp_shift; /* bytes per physical page */
+static addrarith_t hp_per_pp;      /* number of heap pages per physical page */
+static addrarith_t pp_mask;        /* ANDed with a virtual address gives
+				    * base address of physical page
+				    */
+static addrarith_t hp_per_pp_mask; /* ANDed with heap page number gives
+				    * first page number in the physical
+				    * page the heap page belongs to.
+				    */
 #define SAME_PHYSPAGE(a,b) (((a) & pp_mask) == ((b) & pp_mask))
 
 gcspace_t current_space; /* has to be exported because IS_ALIVE depends on it */
 
 static gcspace_t forward_space, previous_space;
-static unsigned current_freepage, current_free;
-static unsigned forward_freepage, forward_free;
-static unsigned last_forward_freepage;
+static pageno_t current_freepage, current_free;
+static pageno_t forward_freepage, forward_free;
+static pageno_t last_forward_freepage;
 
 static Object *current_freep, *forward_freep;
 
@@ -97,16 +105,16 @@ static Object *scanfirst, *scanlast;
 			     && (Object*)(addr) <= scanlast)
 #define IS_SCANNED(addr) ((Object *)(addr) < scanpointer)
 #define MAXRESCAN 10
-static unsigned rescan[MAXRESCAN];
+static pageno_t rescan[MAXRESCAN];
 static int rescanpages = 0;
 static int allscan = 0;
 
-static unsigned stable_queue, stable_tail; /* head and tail of the queue
+static pageno_t stable_queue, stable_tail; /* head and tail of the queue
 					    * of stable pages */
 
 #define DIRTYENTRIES 20
 struct dirty_rec {
-    unsigned pages[DIRTYENTRIES];
+    pageno_t pages[DIRTYENTRIES];
     struct dirty_rec *next;
 };
 
@@ -124,12 +132,6 @@ static void TerminateGC ();
 #define PAGEWORDS      (PAGEBYTES / sizeof (Object))
 #define HEAPPAGEMASK   ~(PAGEBYTES-1)
 
-#ifdef POINTER_CONSTANT_HIGH_BITS
-#  define GC_PCH_BITS POINTER_CONSTANT_HIGH_BITS
-#else
-#  define GC_PCH_BITS 0
-#endif
-
 #ifdef ALIGN_8BYTE
 #  define MAX_OBJECTWORDS       (PAGEWORDS - 1)
 #  define NEEDED_PAGES(size)    (((size) + PAGEWORDS) / PAGEWORDS)
@@ -138,9 +140,9 @@ static void TerminateGC ();
 #  define NEEDED_PAGES(size)    (((size) + PAGEWORDS - 1) / PAGEWORDS)
 #endif
 
-#define MAKE_HEADER(words,type)		((type) << VALBITS | (words))
-#define HEADER_TO_TYPE(header)		((unsigned)(header) >> VALBITS)
-#define HEADER_TO_WORDS(header)		((unsigned)(header) & VALMASK)
+#define MAKE_HEADER(obj,words,type)	(SET(obj, type, words))
+#define HEADER_TO_TYPE(header)		((unsigned)TYPE(header))
+#define HEADER_TO_WORDS(header)		((unsigned)FIXNUM(header))
 
 /* some conversion stuff. PHYSPAGE converts a logical page number into the
  * start address of the physical page the logical page lies on.
@@ -149,7 +151,7 @@ static void TerminateGC ();
  * Note that PAGE_TO_ADDR(0) == 0 if ARRAY_BROKEN is not defined...
  */
 
-#define OBJ_TO_PPADDR(obj) ((gcptr_t)POINTER((Object)(obj)) & pp_mask)
+#define OBJ_TO_PPADDR(obj) ((gcptr_t)POINTER(obj) & pp_mask)
 #define PTR_TO_PPADDR(ptr) ((gcptr_t)(ptr) & pp_mask)
 #define ADDR_TO_PAGE(addr) ((((addr) & HEAPPAGEMASK) / PAGEBYTES) - pagebase)
 #define PAGE_TO_ADDR(page) (((page) + pagebase) * PAGEBYTES)
@@ -225,15 +227,18 @@ static void SetupDirtyList () {
     dirtylist = (struct dirty_rec *) malloc (sizeof (struct dirty_rec));
     if (dirtylist == (struct dirty_rec *)0)
 	Fatal_Error ("SetupDirtyList: unable to allocate memory");
+    bzero ((char *)dirtylist->pages, sizeof (dirtylist->pages));
+    dirtylist->next = (struct dirty_rec *)0;
     dirtyhead = dirtylist;
     dirtyentries = 0;
 }
 
-static void AddDirty (addr) unsigned addr; {
+static void AddDirty (addr) pageno_t addr; {
     struct dirty_rec *p;
 
-    if (dirtylist->pages[(dirtyentries-1) % DIRTYENTRIES] == addr)
-	return;
+    if (dirtyentries != 0 &&
+	dirtylist->pages[(dirtyentries-1) % DIRTYENTRIES] == addr)
+	    return;
     else
 	dirtylist->pages[dirtyentries++ % DIRTYENTRIES] = addr;
 
@@ -241,9 +246,10 @@ static void AddDirty (addr) unsigned addr; {
 	p = (struct dirty_rec *) malloc (sizeof (struct dirty_rec));
 	if (p == (struct dirty_rec *)0)
 	    Fatal_Error ("AddDirty: unable to allocate memory");
+	bzero ((char *)p->pages, sizeof (p->pages));
+	p->next = (struct dirty_rec *)0;
 	dirtylist->next = p;
 	dirtylist = p;
-	p->next = (struct dirty_rec *)0;
     }
 }
 
@@ -269,7 +275,7 @@ static void ReprotectDirty () {
  * to remember pages, set a flag to rescan the whole scan region.
  */
 
-static void RegisterPage (page) unsigned page; {
+static void RegisterPage (page) pageno_t page; {
     if (allscan)
 	return;
 
@@ -298,7 +304,9 @@ static void DetermineCluster (addr, len) gcptr_t *addr; int *len; {
     }
     addr1 = *addr + ((*len) << pp_shift);
 
-    while (type[ADDR_TO_PAGE (addr1)] != OBJECTPAGE) {
+    while (ADDR_TO_PAGE(addr1) <= lastpage &&
+	    space[ADDR_TO_PAGE(addr1)] > 0 &&
+	    type[ADDR_TO_PAGE(addr1)] != OBJECTPAGE) {
 	addr1 += bytes_per_pp;
 	(*len)++;
     }
@@ -347,13 +355,13 @@ static void UnprotectCluster (addr, len) gcptr_t addr; {
 
 /* add one page to the stable set queue */
 
-static void AddQueue (page) unsigned page; {
+static void AddQueue (page) pageno_t page; {
 
-    if (stable_queue != (unsigned)(-1))
+    if (stable_queue != (pageno_t)-1)
 	link[stable_tail] = page;
     else
 	stable_queue = page;
-    link[page] = (unsigned)(-1);
+    link[page] = (pageno_t)-1;
     stable_tail = page;
 }
 
@@ -366,9 +374,9 @@ static void AddQueue (page) unsigned page; {
 static void PromoteStableQueue () {
     Object *p;
     int pcount, size;
-    unsigned start;
+    pageno_t start;
     
-    while (stable_queue != (unsigned)(-1)) {
+    while (stable_queue != (pageno_t)-1) {
 	p = PAGE_TO_OBJ (stable_queue);
 #ifdef ALIGN_8BYTE
 	p++;
@@ -388,21 +396,30 @@ static void PromoteStableQueue () {
 /* calculate the logarithm (base 2) for arguments == 2**n
  */
 
-static Logbase2 (psize) unsigned psize; {
+static Logbase2 (psize) addrarith_t psize; {
     int shift = 0;
 
+#if LONG_BITS-64 == 0
+    if (psize & 0xffffffff00000000) shift += 32;
+    if (psize & 0xffff0000ffff0000) shift += 16;
+    if (psize & 0xff00ff00ff00ff00) shift += 8;
+    if (psize & 0xf0f0f0f0f0f0f0f0) shift += 4;
+    if (psize & 0xcccccccccccccccc) shift += 2;
+    if (psize & 0xaaaaaaaaaaaaaaaa) shift += 1;
+#else
     if (psize & 0xffff0000) shift += 16;
     if (psize & 0xff00ff00) shift += 8;
     if (psize & 0xf0f0f0f0) shift += 4;
     if (psize & 0xcccccccc) shift += 2;
     if (psize & 0xaaaaaaaa) shift += 1;
+#endif
 
     return (shift);
 }
 
 /* return next heap page number, wrap around at the end of the heap. */
 
-static int next (page) unsigned page; {
+static pageno_t next (page) pageno_t page; {
     return ((page < lastpage) ? page+1 : firstpage);
 }
 
@@ -433,9 +450,10 @@ static char *heapmalloc (s) {
  */
 
 Make_Heap (size) {
-    unsigned heapsize = size * 2 * 1024;
-    char *heap;
-    unsigned i;
+    addrarith_t heapsize = size * 2 * 1024;
+    char *heap_ptr, *aligned_heap_ptr;
+    Object heap_obj;
+    pageno_t i;
     
 #ifdef HAS_MPROTECT
     InstallHandler ();
@@ -467,32 +485,33 @@ Make_Heap (size) {
     pp_mask = ~(bytes_per_pp-1);
     pp_shift = Logbase2 (bytes_per_pp);
 
-    heap = heapmalloc (logical_pages*PAGEBYTES+bytes_per_pp-1);
+    heap_ptr = heapmalloc (logical_pages*PAGEBYTES+bytes_per_pp-1);
 
-    if (heap == NULL)
+    if (heap_ptr == NULL)
 	Fatal_Error ("cannot allocate heap (%u KBytes)", size);
     
-    if ((((gcptr_t)heap+heapsize-1) & ~VALMASK) != GC_PCH_BITS)
-	Fatal_Error ("heap too large, %u KBytes max",
-		     (VALMASK - ((gcptr_t)heap & ~GC_PCH_BITS) >> 10));
-    
     /* Align heap at a memory page boundary */
     
-    if ((gcptr_t)heap & (bytes_per_pp-1))
-	heap = (char*)(((gcptr_t)heap+bytes_per_pp) & ~(bytes_per_pp-1));
+    if ((gcptr_t)heap_ptr & (bytes_per_pp-1))
+	aligned_heap_ptr = (char*)(((gcptr_t)heap_ptr+bytes_per_pp)
+	    & ~(bytes_per_pp-1));
+    else
+	aligned_heap_ptr = heap_ptr;
+
+    SET(heap_obj, 0, aligned_heap_ptr);
 
 #ifdef ARRAY_BROKEN    
-    pagebase = ((gcptr_t)POINTER ((Object)(heap))) / PAGEBYTES;
+    pagebase = ((gcptr_t)POINTER (heap_obj)) / PAGEBYTES;
 #endif
-    firstpage = OBJ_TO_PAGE ((Object)heap);
+    firstpage = OBJ_TO_PAGE (heap_obj);
     lastpage = firstpage+logical_pages-1;
     
     space = (gcspace_t *)malloc (logical_pages*sizeof (gcspace_t));
     type = (gcspace_t *)malloc ((logical_pages + 1)*sizeof (gcspace_t));
     pmap = (gcspace_t *)malloc (physical_pages*sizeof (gcspace_t));
-    link = (unsigned *)malloc (logical_pages*sizeof (unsigned));
+    link = (pageno_t *)malloc (logical_pages*sizeof (pageno_t));
     if (!space || !type || !pmap || !link) {
-	free (heap);
+	free (heap_ptr);
 	if (space) free ((char*)space);
 	if (type) free ((char*)type);
 	if (pmap) free ((char*)pmap);
@@ -500,7 +519,9 @@ Make_Heap (size) {
 	Fatal_Error ("cannot allocate heap maps");
     }
 
+    bzero ((char *)type, (logical_pages + 1)*sizeof (gcspace_t));
     bzero ((char *)pmap, physical_pages*sizeof (gcspace_t));
+    bzero ((char *)link, logical_pages*sizeof (unsigned));
     space -= firstpage; /* to index the arrays with the heap page number */
     type -= firstpage;
     type[lastpage+1] = OBJECTPAGE;
@@ -516,7 +537,7 @@ Make_Heap (size) {
     forwarded_pages = 0;
     current_pages = 0;
     protected_pages = 0;
-    stable_queue = (unsigned)(-1);
+    stable_queue = (pageno_t)-1;
     SetupDirtyList ();
 
     current_space = forward_space = previous_space = 3;
@@ -524,33 +545,33 @@ Make_Heap (size) {
 }
 
 /*
- * increment the heap by 1024 kB.
+ * increment the heap by 1024 KB.
  */
 
 static int ExpandHeap (reason) char *reason; {
     int increment = (1024 * 1024 + bytes_per_pp - 1) / bytes_per_pp;
     int incpages = increment * hp_per_pp;
-    unsigned heapinc = incpages * PAGEBYTES;
-    unsigned new_first, inc_first;
-    unsigned new_last, inc_last;
-    int new_logpages, new_physpages;
-    int new_spanpages;
+    addrarith_t heapinc = incpages * PAGEBYTES;
+    pageno_t new_first, inc_first;
+    pageno_t new_last, inc_last;
+    pageno_t new_logpages, new_physpages;
+    pageno_t new_spanpages;
     gcptr_t addr;
     gcspace_t *new_space, *new_type, *new_pmap;
-    unsigned *new_link, i;
-    char *heap;
+    pageno_t *new_link, i;
+    char *heap_ptr, *aligned_heap_ptr;
+    Object heap_obj;
 #ifdef ARRAY_BROKEN
-    unsigned new_pagebase, offset;
-    unsigned new_firstpage, new_lastpage;
+    pageno_t new_pagebase, offset;
+    pageno_t new_firstpage, new_lastpage;
 #else
 #   define offset 0
 #endif
     
-    heap = heapmalloc (heapinc+bytes_per_pp-1);
+    heap_ptr = heapmalloc (heapinc+bytes_per_pp-1);
 
-    if (heap == NULL ||
-	    (((gcptr_t)heap+heapinc-1) & ~VALMASK) != GC_PCH_BITS) {
-	if (Truep (Var_Get (V_Garbage_Collect_Notifyp))) {
+    if (heap_ptr == NULL) {
+	if (Var_Is_True (V_Garbage_Collect_Notifyp)) {
 	    char buf[243];
 	    sprintf(buf, "[Heap expansion failed (%s)]~%%", reason);
  	    Format (Standard_Output_Port, buf,
@@ -562,15 +583,20 @@ static int ExpandHeap (reason) char *reason; {
     
     /* Align heap at a memory page boundary */
     
-    if ((gcptr_t)heap & (bytes_per_pp-1))
-	heap = (char*)(((gcptr_t)heap+bytes_per_pp-1) & ~(bytes_per_pp-1));
+    if ((gcptr_t)heap_ptr & (bytes_per_pp-1))
+	aligned_heap_ptr = (char*)(((gcptr_t)heap_ptr+bytes_per_pp-1)
+	    & ~(bytes_per_pp-1));
+    else
+	aligned_heap_ptr = heap_ptr;
+
+    SET(heap_obj, 0, aligned_heap_ptr);
     
     new_first = firstpage;
     new_last = lastpage;
 
 #ifdef ARRAY_BROKEN
-    new_pagebase = ((gcptr_t)POINTER ((Object)(heap))) / PAGEBYTES;
-    inc_first = 0; /* = OBJ_TO_PAGE ((Object)heap) - new_pagebase */
+    new_pagebase = ((gcptr_t)POINTER (heap_obj)) / PAGEBYTES;
+    inc_first = 0; /* = OBJ_TO_PAGE (heap_obj) - new_pagebase */
 
     new_firstpage = (pagebase > new_pagebase)
 	? new_pagebase : pagebase;
@@ -581,7 +607,7 @@ static int ExpandHeap (reason) char *reason; {
 
     offset = pagebase - new_firstpage;
 #else
-    inc_first = OBJ_TO_PAGE ((Object)heap);
+    inc_first = OBJ_TO_PAGE (heap_obj);
 #endif
 
     inc_last = inc_first+incpages-1;
@@ -601,14 +627,14 @@ static int ExpandHeap (reason) char *reason; {
     new_space = (gcspace_t *)malloc (new_spanpages*sizeof (gcspace_t));
     new_type = (gcspace_t *)malloc ((new_spanpages + 1)*sizeof (gcspace_t));
     new_pmap = (gcspace_t *)malloc (new_physpages*sizeof (gcspace_t));
-    new_link = (unsigned *)malloc (new_spanpages*sizeof (unsigned));
+    new_link = (pageno_t *)malloc (new_spanpages*sizeof (pageno_t));
     if (!new_space || !new_type || !new_pmap || !new_link) {
-	free (heap);
+	free (heap_ptr);
 	if (new_space) free ((char*)new_space);
 	if (new_type) free ((char*)new_type);
 	if (new_pmap) free ((char*)new_pmap);
 	if (new_link) free ((char*)new_link);
-	if (Truep (Var_Get (V_Garbage_Collect_Notifyp))) {
+	if (Var_Is_True (V_Garbage_Collect_Notifyp)) {
  	    Format (Standard_Output_Port, "[Heap expansion failed]~%",
  		    25, 0, (Object *)0);
 	    (void)fflush (stdout);
@@ -644,24 +670,7 @@ static int ExpandHeap (reason) char *reason; {
     new_type[new_spanpages] = OBJECTPAGE;
 #else
     for (i = new_first; i < firstpage; i++) new_space[i] = UNALLOCATED_PAGE;
-    for (i = firstpage; i <= lastpage; i++) {
-	new_space[i] = space[i];
-#ifdef mysterious_mprotect_bug
-    {
-	int j = PHYSPAGE(firstpage) >> pp_shift;
-	
-	if (bcmp(pmap+j,   new_pmap+j, physical_pages * sizeof(gcspace_t))) {
-	    
-	    fprintf(stderr, "bcmp2b %d, pmap+j %x, new_pmap+j %x, size %x\n,
-		    new_space+i %x, space+i %x, size %x", i,
-		    pmap+j, new_pmap+j, physical_pages * sizeof(gcspace_t),
-		    new_space+i, space+i,
-		    (lastpage+1-firstpage) * sizeof (gcspace_t), 0);
-	    abort();
-	}
-    }
-#endif
-    }
+    for (i = firstpage; i <= lastpage; i++) new_space[i] = space[i];
     
     for (i = lastpage+1; i <= new_last; i++) new_space[i] = UNALLOCATED_PAGE;
     for (i = inc_first; i <= inc_last; i++) new_space[i] = FREE_PAGE;
@@ -692,7 +701,7 @@ static int ExpandHeap (reason) char *reason; {
     spanning_pages = new_spanpages;
     physical_pages = new_physpages;
     
-    if (Truep (Var_Get (V_Garbage_Collect_Notifyp))) {
+    if (Var_Is_True (V_Garbage_Collect_Notifyp)) {
  	int a = (logical_pages * PAGEBYTES) >> 10;
 	char buf[243];
 
@@ -709,7 +718,7 @@ static int ExpandHeap (reason) char *reason; {
  * The new page is allocated in current space.
  */
 
-static int ProtectedInRegion (start, npages) unsigned start; {
+static int ProtectedInRegion (start, npages) pageno_t start, npages; {
     gcptr_t beginpage = PHYSPAGE (start);
     gcptr_t endpage = PHYSPAGE (start+npages-1);
 
@@ -722,13 +731,13 @@ static int ProtectedInRegion (start, npages) unsigned start; {
     return (0);
 }
 
-static void AllocPage (npg) {
-    unsigned first_freepage;    /* first free heap page */
-    int cont_free;              /* contiguous free pages */
-    unsigned n, p;
+static void AllocPage (npg) pageno_t npg; {
+    pageno_t first_freepage;    /* first free heap page */
+    pageno_t cont_free;         /* contiguous free pages */
+    pageno_t n, p;
     
     if (current_space != forward_space) {
-	Scanner (1);
+	(void)Scanner ((pageno_t)1);
 	if (!protected_pages)
 	    TerminateGC ();
     } else {
@@ -774,7 +783,7 @@ static void AllocPage (npg) {
 		allocated_pages += npg;
 		current_freepage = next (first_freepage+npg-1);
 		if (ProtectedInRegion (first_freepage, npg))
-		    ScanCluster (PHYSPAGE (first_freepage));
+		    (void)ScanCluster (PHYSPAGE (first_freepage));
 		return;
 	    } else {
 		current_freepage = next (current_freepage);
@@ -793,7 +802,7 @@ static void AllocPage (npg) {
 	return;
     }
     
-    Fatal_Error ("Unable to allocate %u bytes in heap", npg*PAGEBYTES);
+    Fatal_Error ("unable to allocate %lu bytes in heap", npg*PAGEBYTES);
     
     /*NOTREACHED*/
 }
@@ -805,8 +814,9 @@ static void AllocPage (npg) {
  */
 
 Object Alloc_Object (size, type, konst) {
-    register Object obj;
-    register s = ((size+3) >> 2) + 1; /* size in words */
+    Object obj;
+    register addrarith_t s = /* size in words */
+	((size + sizeof(Object) - 1) / sizeof(Object)) + 1;
     int big = 0;
 
     if (GC_Debug) {
@@ -823,7 +833,7 @@ Object Alloc_Object (size, type, konst) {
     
     if (s > current_free) {
 	if (current_free) {
-	    *current_freep = MAKE_HEADER (current_free, T_Freespace);
+	    MAKE_HEADER (*current_freep, current_free, T_Freespace);
 	    current_free = 0;
 	}
 	
@@ -833,18 +843,19 @@ Object Alloc_Object (size, type, konst) {
 	
 #ifdef ALIGN_8BYTE
 	if (s < PAGEWORDS-1)
-	    AllocPage (1);
+	    AllocPage ((pageno_t)1);
 	else {
-	    AllocPage ((s+PAGEWORDS)/PAGEWORDS);
+	    AllocPage ((pageno_t)(s+PAGEWORDS)/PAGEWORDS);
 	    big = 1;
 	}
-	*current_freep++ = MAKE_HEADER (1, T_Align_8Byte);
+	MAKE_HEADER (*current_freep, 1, T_Align_8Byte);
+	current_freep++;
 	current_free--;
 #else
 	if (s < PAGEWORDS)
-	    AllocPage (1);
+	    AllocPage ((pageno_t)1);
 	else {
-	    AllocPage ((s+PAGEWORDS-1)/PAGEWORDS);
+	    AllocPage ((pageno_t)(s+PAGEWORDS-1)/PAGEWORDS);
 	    big = 1;
 	}
 #endif
@@ -855,7 +866,9 @@ Object Alloc_Object (size, type, konst) {
      * in the current heappage.
      */
     
-    *current_freep++ = MAKE_HEADER (s, type);
+    MAKE_HEADER (*current_freep, s, type);
+    current_freep++;
+    *current_freep = Null;
     SET (obj, type, current_freep);
     if (big)
 	current_freep = (Object*)0, current_free = 0;
@@ -863,7 +876,8 @@ Object Alloc_Object (size, type, konst) {
 	current_freep += (s-1), current_free -= s;
 #ifdef ALIGN_8BYTE
     if (!((gcptr_t)current_freep & 7) && current_free) {
-	*current_freep++ = MAKE_HEADER (1, T_Align_8Byte);
+	MAKE_HEADER (*current_freep, 1, T_Align_8Byte);
+	current_freep++;
 	current_free--;
     }
 #endif
@@ -882,8 +896,8 @@ Object Alloc_Object (size, type, konst) {
 
 static void AllocForwardPage (bad) Object bad; {
     Object *badaddr = (Object *)POINTER (bad);
-    int whole_heap = spanning_pages;
-    unsigned tpage;
+    pageno_t whole_heap = spanning_pages;
+    pageno_t tpage;
     
     while (whole_heap--) {
 	if (space[forward_freepage] < previous_space
@@ -914,7 +928,7 @@ static void AllocForwardPage (bad) Object bad; {
 	return;
     }
     
-    Fatal_Error ("unable to allocate forward page in %u KBytes heap",
+    Fatal_Error ("unable to allocate forward page in %lu KBytes heap",
 		 (logical_pages * PAGEBYTES) >> 10);
     
     /*NOTREACHED*/
@@ -926,13 +940,14 @@ static void AllocForwardPage (bad) Object bad; {
  */
 
 Visit (cp) register Object *cp; {
-    register unsigned page = OBJ_TO_PAGE (*cp);
+    register pageno_t page = OBJ_TO_PAGE (*cp);
     register Object *obj_ptr = (Object *)POINTER (*cp);
     int tag = TYPE (*cp);
     int konst = ISCONST (*cp);
-    int objwords, objpages, pcount;
+    addrarith_t objwords;
+    pageno_t objpages, pcount;
     gcptr_t ffreep, pageaddr = 0;
-    short outside;
+    int outside;
     
     /* if the Visit function is called via the REVIVE_OBJ macro and we are
      * not inside an incremental collection, exit immediately.
@@ -947,11 +962,9 @@ Visit (cp) register Object *cp; {
 	return;
 
     if (space[page] != previous_space) {
-
-
 	char buf[100];
-	sprintf (buf, "Visit: object not in prev space at 0x%x ('%s')",
-		     obj_ptr, Types[tag].name);
+	sprintf (buf, "Visit: object not in prev space at 0x%lx ('%s') %d %d",
+	    obj_ptr, Types[tag].name, space[page], previous_space);
 	Panic (buf);
     }
     
@@ -963,7 +976,7 @@ Visit (cp) register Object *cp; {
     if (WAS_FORWARDED (*cp)) {
 	if (pageaddr != 0)
 	    PROTECT (pageaddr);
-	*cp = MAKEOBJ (tag, *obj_ptr);
+	MAKEOBJ (*cp, tag, POINTER(*obj_ptr));
 	if (konst)
 	    SETCONST (*cp);
 	return;
@@ -1000,11 +1013,11 @@ Visit (cp) register Object *cp; {
 		&& !SAME_PHYSPAGE ((gcptr_t)obj_ptr, ffreep)) {
 		
 		UNPROTECT (ffreep);
-		*forward_freep = MAKE_HEADER (forward_free, T_Freespace);
+		MAKE_HEADER (*forward_freep, forward_free, T_Freespace);
 		forward_free = 0;
 		PROTECT (ffreep);
 	    } else {
-		*forward_freep = MAKE_HEADER (forward_free, T_Freespace);
+		MAKE_HEADER (*forward_freep, forward_free, T_Freespace);
 		forward_free = 0;
 	    }
 	}
@@ -1015,7 +1028,8 @@ Visit (cp) register Object *cp; {
 #ifdef ALIGN_8BYTE
 	if (outside && IS_PROTECTED (ffreep))
 	    UNPROTECT (ffreep);
-	*forward_freep++ = MAKE_HEADER (1, T_Align_8Byte);
+	MAKE_HEADER (*forward_freep, 1, T_Align_8Byte);
+	forward_freep++;
 	forward_free--;
 	goto do_forward;
 #endif
@@ -1028,14 +1042,15 @@ Visit (cp) register Object *cp; {
 do_forward:
 #endif
     if (tag == T_Control_Point) {
-	CONTROL (POINTER (*cp))->reloc =
+	CONTROL (*cp)->reloc =
 	    (char*)(forward_freep + 1) - (char*)obj_ptr;
     }
 	
-    *forward_freep++ = MAKE_HEADER (objwords, tag);
-    bcopy ((char*)obj_ptr, (char*)forward_freep, (objwords-1)*sizeof (int));
+    MAKE_HEADER (*forward_freep, objwords, tag);
+    forward_freep++;
+    bcopy ((char*)obj_ptr, (char*)forward_freep, (objwords-1)*sizeof(Object));
     SET (*obj_ptr, T_Broken_Heart, forward_freep);
-    *cp = MAKEOBJ (tag, forward_freep);
+    MAKEOBJ (*cp, tag, forward_freep);
     if (konst)
 	SETCONST (*cp);
     forward_freep += (objwords - 1);
@@ -1043,7 +1058,8 @@ do_forward:
 	
 #ifdef ALIGN_8BYTE
     if (!((gcptr_t)forward_freep & 7) && forward_free) {
-	*forward_freep++ = MAKE_HEADER (1, T_Align_8Byte);
+	MAKE_HEADER (*forward_freep, 1, T_Align_8Byte);
+	forward_freep++;
 	forward_free--;
     }
 #endif
@@ -1063,8 +1079,9 @@ do_forward:
  */
     
 static void ScanPage (currentp, nextcp) Object *currentp, *nextcp; {
-    Object *cp = currentp;
-    int len, m, n, t;
+    Object *cp = currentp, obj;
+    addrarith_t len, m, n;
+    int t;
     
     while (cp < nextcp && (cp != forward_freep || forward_free == 0)) {
 	t = HEADER_TO_TYPE (*cp);
@@ -1075,71 +1092,71 @@ static void ScanPage (currentp, nextcp) Object *currentp, *nextcp; {
 	 * the type of the object, len its length inclusive header in
 	 * words.
 	 */
+
+	SET(obj, t, cp);
 	
-	if (Types[t].visit != NOFUNC)
-	    switch (t) {
-	    case T_Symbol:
-		Visit (&SYMBOL((Object)cp)->next);
-		Visit (&SYMBOL((Object)cp)->name);
-		Visit (&SYMBOL((Object)cp)->value);
-		Visit (&SYMBOL((Object)cp)->plist);
-		break;
-		
-	    case T_Pair:
-	    case T_Environment:
-		Visit (&PAIR((Object)cp)->car);
-		Visit (&PAIR((Object)cp)->cdr);
-		break;
-		
-	    case T_Vector:
-		for (n = 0, m = VECTOR((Object)cp)->size; n < m; n++ )
-		    Visit (&VECTOR((Object)cp)->data[n]);
-		break;
-		
-	    case T_Compound:
-		Visit (&COMPOUND((Object)cp)->closure);
-		Visit (&COMPOUND((Object)cp)->env);
-		Visit (&COMPOUND((Object)cp)->name);
-		break;
-		
-	    case T_Control_Point:
-		(CONTROL((Object)cp)->delta) += CONTROL((Object)cp)->reloc;
+	switch (t) {
+	case T_Symbol:
+	    Visit (&SYMBOL(obj)->next);
+	    Visit (&SYMBOL(obj)->name);
+	    Visit (&SYMBOL(obj)->value);
+	    Visit (&SYMBOL(obj)->plist);
+	    break;
 		
+	case T_Pair:
+	case T_Environment:
+	    Visit (&PAIR(obj)->car);
+	    Visit (&PAIR(obj)->cdr);
+	    break;
+	    
+	case T_Vector:
+	    for (n = 0, m = VECTOR(obj)->size; n < m; n++ )
+		Visit (&VECTOR(obj)->data[n]);
+	    break;
+	    
+	case T_Compound:
+	    Visit (&COMPOUND(obj)->closure);
+	    Visit (&COMPOUND(obj)->env);
+	    Visit (&COMPOUND(obj)->name);
+	    break;
+	    
+	case T_Control_Point:
+	    (CONTROL(obj)->delta) += CONTROL(obj)->reloc;
+	    
 #ifdef USE_ALLOCA
-		Visit_GC_List (CONTROL((Object)cp)->gclist, CONTROL((Object)cp)->delta);
+	    Visit_GC_List (CONTROL(obj)->gclist, CONTROL(obj)->delta);
 #else
-		Visit (&CONTROL((Object)cp)->gcsave);
+	    Visit (&CONTROL(obj)->gcsave);
 #endif
-		Visit_Wind (CONTROL((Object)cp)->firstwind,
-			    (CONTROL((Object)cp)->delta) );
+	    Visit_Wind (CONTROL(obj)->firstwind,
+			(CONTROL(obj)->delta) );
+	    
+	    Visit (&CONTROL(obj)->env);
+	    break;
 		
-		Visit (&CONTROL((Object)cp)->env);
-		break;
-		
-	    case T_Promise:
-		Visit (&PROMISE((Object)cp)->env);
-		Visit (&PROMISE((Object)cp)->thunk);
-		break;
-		
-	    case T_Port:
-		Visit (&PORT((Object)cp)->name);
-		break;
-		
-	    case T_Autoload:
-		Visit (&AUTOLOAD((Object)cp)->files);
-		Visit (&AUTOLOAD((Object)cp)->env);
-		break;
-		
-	    case T_Macro:
-		Visit (&MACRO((Object)cp)->body);
-		Visit (&MACRO((Object)cp)->name);
-		break;
-	    default: {
-		Object o;
-		SET(o, t, cp);
-		(*Types[t].visit) (&o, Visit);
-	    }
-	    }		
+	case T_Promise:
+	    Visit (&PROMISE(obj)->env);
+	    Visit (&PROMISE(obj)->thunk);
+	    break;
+	    
+	case T_Port:
+	    Visit (&PORT(obj)->name);
+	    break;
+	    
+	case T_Autoload:
+	    Visit (&AUTOLOAD(obj)->files);
+	    Visit (&AUTOLOAD(obj)->env);
+	    break;
+	    
+	case T_Macro:
+	    Visit (&MACRO(obj)->body);
+	    Visit (&MACRO(obj)->name);
+	    break;
+
+	default: 
+	    if (Types[t].visit)
+		(Types[t].visit) (&obj, Visit);
+	}
 	cp += (len - 1);
     }
 }
@@ -1164,14 +1181,16 @@ static void RescanPages () {
 }
     
 static int ScanCluster (addr) gcptr_t addr; {
-    register unsigned page, lastpage;
-    int npages = 0;
+    register pageno_t page, lastpage;
+    pageno_t npages;
+    int n = 0;
 	
     scanning = 1;
-    DetermineCluster (&addr, &npages);
+    DetermineCluster (&addr, &n);
+    npages = n;
     scanfirst = (Object *)addr;
     scanlast = (Object *)(addr + (npages << pp_shift) - sizeof (Object));
-    UnprotectCluster (scanfirst, npages);
+    UnprotectCluster ((gcptr_t)scanfirst, (int)npages);
 
  rescan_cluster:
     lastpage = ADDR_TO_PAGE ((gcptr_t)scanlast);
@@ -1203,10 +1222,10 @@ static int ScanCluster (addr) gcptr_t addr; {
 }
 
 
-static int Scanner (npages) {
+static int Scanner (npages) pageno_t npages; {
     register gcptr_t addr, lastaddr;
-    int spages;
-    int scanned = 0;
+    pageno_t spages;
+    pageno_t scanned = 0;
     
     while (npages > 0 && protected_pages) {
 	lastaddr = PAGE_TO_ADDR (lastpage);
@@ -1244,7 +1263,10 @@ static void PagefaultHandler (sig, code, scp) struct sigcontext *scp; {
 #ifdef SIGSEGV_AIX
 
 static void PagefaultHandler (sig, code, scp) struct sigcontext *scp; {
-    char *addr = scp->sc_jmpbuf.jmp_context.except[3];
+    char *addr = (char *)scp->sc_jmpbuf.jmp_context.except[3];
+    /*
+     * Or should that be .jmp_context.o_vaddr?
+     */
 
 #else
 #ifdef SIGSEGV_SIGINFO
@@ -1271,12 +1293,12 @@ static void PagefaultHandler (sig, code, scp) struct sigcontext *scp; {
 #endif
 #endif
 
-    unsigned page;
+    pageno_t page;
     gcptr_t ppage;
     char *errmsg = 0;
 
 #ifdef SIGSEGV_AIX
-    if (scp->sc_jmpbuf.jmp_context.except[0] != addr)
+    if ((char *)scp->sc_jmpbuf.jmp_context.except[0] != addr)
 	Panic ("except");
 #endif
 
@@ -1309,7 +1331,7 @@ static void PagefaultHandler (sig, code, scp) struct sigcontext *scp; {
     }
 
     GC_In_Progress = 1;
-    ScanCluster (ppage);
+    (void)ScanCluster (ppage);
     GC_In_Progress = 0;
 #ifdef SIGSEGV_AIX
     InstallHandler ();
@@ -1322,7 +1344,7 @@ InstallHandler () {
     struct sigaction sact;
     sigset_t mask;
 
-    sact.sa_handler = PagefaultHandler;
+    sact.sa_handler = (void (*)())PagefaultHandler;
     sigemptyset (&mask);
     sact.sa_mask = mask;
     sact.sa_flags = SA_SIGINFO;
@@ -1335,31 +1357,6 @@ InstallHandler () {
 }
 #endif
 
-#ifdef mysterious_mprotect_bug
-z2(s, pmap) char* s; gcspace_t pmap[]; {
-    int i, j = 0;
-    
-    for (i = firstpage; i <= lastpage; ++i) {
-	switch (IS_PROTECTED(PAGE_TO_ADDR(i))) {
-	case 1:
-	    j++;
-	    break;
-	case 0:
-	    break;
-	default:
-	    fprintf(stderr, "IS_PROTECTED(PAGE_TO_ADDR(%d)) = %d\n", i,
-		    IS_PROTECTED(PAGE_TO_ADDR(i)));
-	}
-    }
-    
-    if ((j/=8) != protected_pages) {
-	fprintf(stderr, "%s: %d pages still protected, protected_pages: %d\n",
-		s, j, protected_pages);
-	abort();
-    }
-}
-#endif
-
 static void TerminateGC () {
     int save_force_total;
     
@@ -1373,7 +1370,7 @@ static void TerminateGC () {
     current_pages = 0;
 
     if (forward_free) {
-	*forward_freep = MAKE_HEADER (forward_free, T_Freespace);
+	MAKE_HEADER (*forward_freep, forward_free, T_Freespace);
 	forward_free = 0;
     }
     forward_freep = (Object *)0;
@@ -1382,10 +1379,11 @@ static void TerminateGC () {
     GC_In_Progress = 0;
     Enable_Interrupts;
 
-    if (Truep (Var_Get (V_Garbage_Collect_Notifyp)) && !GC_Debug) {
+    if (Var_Is_True (V_Garbage_Collect_Notifyp) && !GC_Debug) {
  	int foo = percent - HEAPPERCENT (allocated_pages);
- 	Object bar = Make_Fixnum (foo);
-
+ 	Object bar;
+	
+	bar = Make_Integer (foo);
 	if (!incomplete_msg)
 	    Format (Standard_Output_Port, "[", 1, 0, (Object *)0);
 
@@ -1407,7 +1405,8 @@ static void TerminateGC () {
 	    P_Collect ();
 	tuneable_force_total = save_force_total;
 	if (HEAPPERCENT (allocated_pages) >= tuneable_newly_expand)
-	    ExpandHeap ("after full collection");
+	    /* return value should not be ignore here: */
+	    (void)ExpandHeap ("after full collection");
     }
 }
 
@@ -1421,7 +1420,7 @@ static void Finish_Collection () {
 	     addr += bytes_per_pp) {
 
 	    if (IS_PROTECTED (addr)) {
-		ScanCluster (addr);
+		(void)ScanCluster (addr);
 		if (protected_pages == 0) TerminateGC ();
 	    }
 	}
@@ -1432,9 +1431,10 @@ static void Finish_Collection () {
 
 
 static void General_Collect (initiate) {
-    unsigned fpage, free_fpages, i;
-    unsigned page;
-    unsigned fregion_pages;
+    pageno_t fpage, free_fpages, i;
+    pageno_t page;
+    pageno_t fregion_pages;
+    Object obj;
 
     if (!Interpreter_Initialized)
 	Fatal_Error ("Out of heap space (increase heap size)");
@@ -1453,7 +1453,7 @@ static void General_Collect (initiate) {
     percent = HEAPPERCENT (allocated_pages);
     old_logical_pages = logical_pages;
 
-    if (Truep (Var_Get (V_Garbage_Collect_Notifyp)) && !GC_Debug) {
+    if (Var_Is_True (V_Garbage_Collect_Notifyp) && !GC_Debug) {
 	if (initiate) {
 	    Format (Standard_Output_Port, "[Garbage collecting...]~%",
 		    25, 0, (Object *)0);
@@ -1473,7 +1473,7 @@ static void General_Collect (initiate) {
     /* discard any remaining portion of the current heap page */
 
     if (current_free) {
-        *current_freep = MAKE_HEADER (current_free, T_Freespace);
+        MAKE_HEADER (*current_freep, current_free, T_Freespace);
         current_free = 0;
     }
 
@@ -1498,7 +1498,8 @@ static void General_Collect (initiate) {
         fpage = next (fpage);
     }
     current_freep = (Object *)PHYSPAGE (fpage);
-    current_freepage = OBJ_TO_PAGE (current_freep);
+    SET(obj, 0, current_freep);
+    current_freepage = OBJ_TO_PAGE (obj);
 
     /* advance spaces. Then forward all objects directly accessible
      * via the global GC lists and the WIND list.
@@ -1518,7 +1519,7 @@ static void General_Collect (initiate) {
      */
  
     page = stable_queue;
-    while (page != (unsigned)(-1)) {
+    while (page != (pageno_t)-1) {
         ProtectCluster (PHYSPAGE (page), 0);
         page = link[page];
     }
@@ -1527,7 +1528,8 @@ static void General_Collect (initiate) {
 	Finish_Collection ();
     } else 
 	if (HEAPPERCENT (forwarded_pages) > tuneable_force_expand)
-            ExpandHeap ("large stable set");
+	    /* return value should not be ignored here: */
+            (void)ExpandHeap ("large stable set");
     
     GC_In_Progress = 0;
     return;
@@ -1548,7 +1550,7 @@ Object P_Collect_Incremental () {
 	}
     } else {
 	if (current_space != forward_space) {
-	    Scanner (1);
+	    (void)Scanner ((pageno_t)1);
 	    GC_In_Progress = 0;
 	    if (protected_pages == 0)
 		TerminateGC ();
@@ -1606,7 +1608,7 @@ Object Internal_GC_Status (strat, flags) {
 #ifdef HAS_MPROTECT
 	if (inc_collection) {
 	    cell = Cons (Sym_Incremental_GC, Null);
-	    (void)P_Setcdr (list, cell);
+	    (void)P_Set_Cdr (list, cell);
 	}
 #endif
 	break;
@@ -1615,7 +1617,7 @@ Object Internal_GC_Status (strat, flags) {
 #ifdef HAS_MPROTECT
 	    inc_collection = 1;
 	    cell = Cons (Sym_Incremental_GC, Null);
-	    (void)P_Setcdr (list, cell);
+	    (void)P_Set_Cdr (list, cell);
 #endif
 	} else inc_collection = 0;
 	break;
