@@ -1,7 +1,7 @@
-/* Build-in primitives, Define_Primitive
+/* Built-in primitives, Define_Primitive().
  */
 
-#include "scheme.h"
+#include "kernel.h"
 
 struct Prim_Init {
     Object (*fun)();
@@ -21,36 +21,40 @@ struct Prim_Init {
     P_Eq,                "eq?",                            2, 2,    EVAL,
     P_Eqv,               "eqv?",                           2, 2,    EVAL,
     P_Equal,             "equal?",                         2, 2,    EVAL,
+    P_Empty_List_Is_False, "empty-list-is-false-for-backward-compatibility",
+							   1, 1,    EVAL,
 
     /* char.c:
      */
     P_Charp,             "char?",                          1, 1,    EVAL,
     P_Char_To_Integer,   "char->integer",                  1, 1,    EVAL,
     P_Integer_To_Char,   "integer->char",                  1, 1,    EVAL,
-    P_Char_Upper_Case,   "char-upper-case?",               1, 1,    EVAL,
-    P_Char_Lower_Case,   "char-lower-case?",               1, 1,    EVAL,
-    P_Char_Alphabetic,   "char-alphabetic?",               1, 1,    EVAL,
-    P_Char_Numeric,      "char-numeric?",                  1, 1,    EVAL,
-    P_Char_Whitespace,   "char-whitespace?",               1, 1,    EVAL,
+    P_Char_Upper_Casep,  "char-upper-case?",               1, 1,    EVAL,
+    P_Char_Lower_Casep,  "char-lower-case?",               1, 1,    EVAL,
+    P_Char_Alphabeticp,  "char-alphabetic?",               1, 1,    EVAL,
+    P_Char_Numericp,     "char-numeric?",                  1, 1,    EVAL,
+    P_Char_Whitespacep,  "char-whitespace?",               1, 1,    EVAL,
     P_Char_Upcase,       "char-upcase",                    1, 1,    EVAL,
     P_Char_Downcase,     "char-downcase",                  1, 1,    EVAL,
-    P_Chr_Eq,            "char=?",                         2, 2,    EVAL,
-    P_Chr_Less,          "char<?",                         2, 2,    EVAL,
-    P_Chr_Greater,       "char>?",                         2, 2,    EVAL,
-    P_Chr_Eq_Less,       "char<=?",                        2, 2,    EVAL,
-    P_Chr_Eq_Greater,    "char>=?",                        2, 2,    EVAL,
-    P_Chr_CI_Eq,         "char-ci=?",                      2, 2,    EVAL,
-    P_Chr_CI_Less,       "char-ci<?",                      2, 2,    EVAL,
-    P_Chr_CI_Greater,    "char-ci>?",                      2, 2,    EVAL,
-    P_Chr_CI_Eq_Less,    "char-ci<=?",                     2, 2,    EVAL,
-    P_Chr_CI_Eq_Greater, "char-ci>=?",                     2, 2,    EVAL,
+    P_Char_Eq,           "char=?",                         2, 2,    EVAL,
+    P_Char_Less,         "char<?",                         2, 2,    EVAL,
+    P_Char_Greater,      "char>?",                         2, 2,    EVAL,
+    P_Char_Eq_Less,      "char<=?",                        2, 2,    EVAL,
+    P_Char_Eq_Greater,   "char>=?",                        2, 2,    EVAL,
+    P_Char_CI_Eq,        "char-ci=?",                      2, 2,    EVAL,
+    P_Char_CI_Less,      "char-ci<?",                      2, 2,    EVAL,
+    P_Char_CI_Greater,   "char-ci>?",                      2, 2,    EVAL,
+    P_Char_CI_Eq_Less,   "char-ci<=?",                     2, 2,    EVAL,
+    P_Char_CI_Eq_Greater,"char-ci>=?",                     2, 2,    EVAL,
 
     /* cont.c:
      */
     P_Control_Pointp,    "control-point?",                 1, 1,    EVAL,
-    P_Call_CC,           "call-with-current-continuation", 1, 1,    EVAL,
+    P_Call_With_Current_Continuation,
+			 "call-with-current-continuation", 1, 1,    EVAL,
     P_Dynamic_Wind,      "dynamic-wind",                   3, 3,    EVAL,
-    P_Control_Point_Env, "control-point-environment",      1, 1,    EVAL,
+    P_Control_Point_Environment,
+			 "control-point-environment",      1, 1,    EVAL,
 
     /* debug.c:
      */
@@ -70,7 +74,8 @@ struct Prim_Init {
     P_Define,            "define",                         1, MANY, NOEVAL,
     P_Define_Macro,      "define-macro",                   1, MANY, NOEVAL,
     P_Set,               "set!",                           2, 2,    NOEVAL,
-    P_Env_List,          "environment->list",              1, 1,    EVAL,
+    P_Environment_To_List,
+			 "environment->list",              1, 1,    EVAL,
     P_Boundp,            "bound?",                         1, 1,    EVAL,
 
     /* error.c:
@@ -78,8 +83,14 @@ struct Prim_Init {
     P_Error,             "error",                          2, MANY, VARARGS,
     P_Reset,             "reset",                          0, 0,    EVAL,
 
+    /* exception.c:
+     */
+    P_Disable_Interrupts,"disable-interrupts",             0, 0,    EVAL,
+    P_Enable_Interrupts, "enable-interrupts",              0, 0,    EVAL,
+
     /* feature.c:
      */
+    P_Features,          "features",                       0, 0,    EVAL,
     P_Featurep,          "feature?",                       1, 1,    EVAL,
     P_Provide,           "provide",                        1, 1,    EVAL,
     P_Require,           "require",                        1, 3,    VARARGS,
@@ -87,14 +98,21 @@ struct Prim_Init {
     /* heap.c:
      */
     P_Collect,           "collect",                        0, 0,    EVAL,
+    P_Garbage_Collect_Status, "garbage-collect-status",    0, 2,    VARARGS,
+#ifdef GENERATIONAL_GC
+    P_Collect_Incremental, "collect-incremental",          0, 0,    EVAL,
+#endif
+
 
     /* io.c:
      */
     P_Port_File_Name,    "port-file-name",                 1, 1,    EVAL,
     P_Port_Line_Number,  "port-line-number",               1, 1,    EVAL,
     P_Eof_Objectp,       "eof-object?",                    1, 1,    EVAL,
-    P_Curr_Input_Port,   "current-input-port",             0, 0,    EVAL,
-    P_Curr_Output_Port,  "current-output-port",            0, 0,    EVAL,
+    P_Current_Input_Port,
+			 "current-input-port",             0, 0,    EVAL,
+    P_Current_Output_Port,
+			 "current-output-port",            0, 0,    EVAL,
     P_Input_Portp,       "input-port?",                    1, 1,    EVAL,
     P_Output_Portp,      "output-port?",                   1, 1,    EVAL,
     P_Open_Input_File,   "open-input-file",                1, 1,    EVAL,
@@ -102,10 +120,10 @@ struct Prim_Init {
     P_Open_Input_Output_File, "open-input-output-file",    1, 1,    EVAL,
     P_Close_Input_Port,  "close-input-port",               1, 1,    EVAL,
     P_Close_Output_Port, "close-output-port",              1, 1,    EVAL,
-    P_With_Input,        "with-input-from-file",           2, 2,    EVAL,
-    P_With_Output,       "with-output-to-file",            2, 2,    EVAL,
-    P_Call_With_Input,   "call-with-input-file",           2, 2,    EVAL,
-    P_Call_With_Output,  "call-with-output-file",          2, 2,    EVAL,
+    P_With_Input_From_File,    "with-input-from-file",     2, 2,    EVAL,
+    P_With_Output_To_File,     "with-output-to-file",      2, 2,    EVAL,
+    P_Call_With_Input_File,    "call-with-input-file",     2, 2,    EVAL,
+    P_Call_With_Output_File,   "call-with-output-file",    2, 2,    EVAL,
     P_Open_Input_String, "open-input-string",              1, 1,    EVAL,
     P_Open_Output_String,"open-output-string",             0, 0,    EVAL,
     P_Tilde_Expand,      "tilde-expand",                   1, 1,    EVAL,
@@ -155,8 +173,8 @@ struct Prim_Init {
     P_Nullp,             "null?",                          1, 1,    EVAL,
     P_Pairp,             "pair?",                          1, 1,    EVAL,
     P_Listp,             "list?",                          1, 1,    EVAL,
-    P_Setcar,            "set-car!",                       2, 2,    EVAL,
-    P_Setcdr,            "set-cdr!",                       2, 2,    EVAL,
+    P_Set_Car,           "set-car!",                       2, 2,    EVAL,
+    P_Set_Cdr,           "set-cdr!",                       2, 2,    EVAL,
     P_Assq,              "assq",                           2, 2,    EVAL,
     P_Assv,              "assv",                           2, 2,    EVAL,
     P_Assoc,             "assoc",                          2, 2,    EVAL,
@@ -194,6 +212,8 @@ struct Prim_Init {
     P_Evenp,             "even?",                          1, 1,    EVAL,
     P_Exactp,            "exact?",                         1, 1,    EVAL,
     P_Inexactp,          "inexact?",                       1, 1,    EVAL,
+    P_Exact_To_Inexact,  "exact->inexact",                 1, 1,    EVAL,
+    P_Inexact_To_Exact,  "inexact->exact",                 1, 1,    EVAL,
     P_Generic_Less,      "<",                              1, MANY, VARARGS,
     P_Generic_Greater,   ">",                              1, MANY, VARARGS,
     P_Generic_Equal,     "=",                              1, MANY, VARARGS,
@@ -254,7 +274,8 @@ struct Prim_Init {
     P_Eval,              "eval",                           1, 2,    VARARGS,
     P_Apply,             "apply",                          2, MANY, VARARGS,
     P_Lambda,            "lambda",                         2, MANY, NOEVAL,
-    P_Procedure_Env,     "procedure-environment",          1, 1,    EVAL,
+    P_Procedure_Environment,
+			 "procedure-environment",          1, 1,    EVAL,
     P_Procedure_Lambda,  "procedure-lambda",               1, 1,    EVAL,
     P_Map,               "map",                            2, MANY, VARARGS,
     P_For_Each,          "for-each",                       2, MANY, VARARGS,
@@ -267,16 +288,18 @@ struct Prim_Init {
     P_Delay,             "delay",                          1, 1,    NOEVAL,
     P_Force,             "force",                          1, 1,    EVAL,
     P_Promisep,          "promise?",                       1, 1,    EVAL,
-    P_Promise_Env,       "promise-environment",            1, 1,    EVAL,
+    P_Promise_Environment,
+			 "promise-environment",            1, 1,    EVAL,
 
     /* read.c:
      */
-    P_Clear_Input_Port,  "clear-input-port",               0, 1,    EVAL,
+    P_Clear_Input_Port,  "clear-input-port",               0, 1,    VARARGS,
     P_Read,              "read",                           0, 1,    VARARGS,
     P_Read_Char,         "read-char",                      0, 1,    VARARGS,
     P_Read_String,       "read-string",                    0, 1,    VARARGS,
     P_Unread_Char,       "unread-char",                    1, 2,    VARARGS,
     P_Peek_Char,         "peek-char",                      0, 1,    VARARGS,
+    P_Char_Readyp,       "char-ready?",                    0, 1,    VARARGS,
 
     /* special.c:
      */
@@ -311,16 +334,17 @@ struct Prim_Init {
     P_String_To_List,    "string->list",                   1, 1,    EVAL,
     P_String_Fill,       "string-fill!",                   2, 2,    EVAL,
     P_Substring_Fill,    "substring-fill!",                4, 4,    EVAL,
-    P_Str_Eq,            "string=?",                       2, 2,    EVAL,
-    P_Str_Less,          "string<?",                       2, 2,    EVAL,
-    P_Str_Greater,       "string>?",                       2, 2,    EVAL,
-    P_Str_Eq_Less,       "string<=?",                      2, 2,    EVAL,
-    P_Str_Eq_Greater,    "string>=?",                      2, 2,    EVAL,
-    P_Str_CI_Eq,         "string-ci=?",                    2, 2,    EVAL,
-    P_Str_CI_Less,       "string-ci<?",                    2, 2,    EVAL,
-    P_Str_CI_Greater,    "string-ci>?",                    2, 2,    EVAL,
-    P_Str_CI_Eq_Less,    "string-ci<=?",                   2, 2,    EVAL,
-    P_Str_CI_Eq_Greater, "string-ci>=?",                   2, 2,    EVAL,
+    P_String_Eq,         "string=?",                       2, 2,    EVAL,
+    P_String_Less,       "string<?",                       2, 2,    EVAL,
+    P_String_Greater,    "string>?",                       2, 2,    EVAL,
+    P_String_Eq_Less,    "string<=?",                      2, 2,    EVAL,
+    P_String_Eq_Greater, "string>=?",                      2, 2,    EVAL,
+    P_String_CI_Eq,      "string-ci=?",                    2, 2,    EVAL,
+    P_String_CI_Less,    "string-ci<?",                    2, 2,    EVAL,
+    P_String_CI_Greater, "string-ci>?",                    2, 2,    EVAL,
+    P_String_CI_Eq_Less, "string-ci<=?",                   2, 2,    EVAL,
+    P_String_CI_Eq_Greater,
+			 "string-ci>=?",                   2, 2,    EVAL,
     P_Substringp,        "substring?",                     2, 2,    EVAL,
     P_CI_Substringp,     "substring-ci?",                  2, 2,    EVAL,
 
@@ -337,7 +361,6 @@ struct Prim_Init {
     /* type.c:
      */
     P_Type,              "type",                           1, 1,    EVAL,
-    P_Voidp,             "void?",                          1, 1,    EVAL,
 
     /* vector.c:
      */
@@ -374,14 +397,12 @@ Init_Prim () {
     Memoize_Frame (frame);
 }
 
-/* Not used by the interpreter kernel (lint may complain).
- */
-Define_Primitive (fun, name, min, max, disc) Object (*fun)(); char *name;
+Define_Primitive (fun, name, min, max, disc) Object (*fun)(); const char *name;
 	enum discipline disc; {
     Object prim, sym, frame;
     GC_Node2;
 
-    Error_Tag = "define-primitive";
+    Set_Error_Tag ("define-primitive");
     prim = Make_Primitive (fun, name, min, max, disc);
     sym = Null;
     GC_Link2 (prim, sym);
