@@ -1,4 +1,5 @@
-/* Eval, funcall, apply, map, lambda, macro, etc.
+/* Eval, funcall, apply, map, lambda, etc.  The main-loop of the
+ * Scheme evaluator.
  */
 
 #include "kernel.h"
@@ -9,7 +10,21 @@
 #  define MAX_ARGS_ON_STACK  8
 #endif
 
-const char *Error_Tag;
+#define Get_Arglist_Length(_cnt, _lst, _x) \
+    for (_cnt = 0, _x = _lst; TYPE(_x) == T_Pair; _x = Cdr (_x), _cnt++) \
+	; \
+    if (!Nullp(_x)) \
+	Primitive_Error("improper argument list"); \
+
+#define Funcall_Switch(t,func,args,eval) \
+    if (t == T_Primitive) {\
+	return Funcall_Primitive (func, args, eval);\
+    } else if (t == T_Compound) {\
+	return Funcall_Compound (func, args, eval);\
+    } else if (t == T_Control_Point) {\
+	Funcall_Control_Point (func, args, eval);\
+    } else Primitive_Error ("application of non-procedure: ~s", func);\
+
 
 /* Tail_Call indicates whether we are executing the last form in a
  * sequence of forms.  If it is true and we are about to call a compound
@@ -23,7 +38,7 @@ Object Sym_Lambda,
 
 static Object tc_fun, tc_argl, tc_env;
 
-Object Macro_Expand();
+Object Macro_Expand(), Funcall_Primitive(), Funcall_Compound();
 
 Init_Proc () {
     Define_Symbol (&Sym_Lambda, "lambda");
@@ -84,7 +99,8 @@ Object Make_Primitive (fun, name, min, max, disc) Object (*fun)();
 Object Eval (form) Object form; {
     register t;
     register struct S_Symbol *sym;
-    Object fun, binding, args, ret;
+    Object fun, binding, ret;
+    static unsigned tick;
     GC_Node;
     TC_Prolog;
 
@@ -92,7 +108,7 @@ again:
     t = TYPE(form);
     if (t == T_Symbol) {
 	sym = SYMBOL(form);
-	if (EQ(sym->value,Unbound)) {
+	if (TYPE(sym->value) == T_Unbound) {
 	    binding = Lookup_Symbol (form, 1);
 	    sym->value = Cdr (binding);
 	}
@@ -108,22 +124,30 @@ again:
 	    Primitive_Error ("unevaluable object: ~s", form);
 	return form;
     }
-    if (Stack_Size () > Max_Stack)
-	Uncatchable_Error ("Out of stack space");
-    GC_Link (form);
-    TC_Disable;
-    fun = Eval (Car (form));
-    TC_Enable;
-    args = Cdr (form);
-    Check_List (args);
-    if (TYPE(fun) == T_Macro) {
-	form = Macro_Expand (fun, args);
+    if ((tick++ & 7) == 0)
+	if (Stack_Size () > Max_Stack)
+	    Uncatchable_Error ("Out of stack space");
+    /*
+     * Avoid recursive Eval() for the most common case:
+     */
+    fun = Car (form);
+    if (TYPE(fun) != T_Symbol ||
+	    (fun = SYMBOL(fun)->value, TYPE(fun) == T_Unbound) ||
+	    TYPE(fun) == T_Autoload) {
+	GC_Link (form);
+	TC_Disable;
+	fun = Eval (Car (form));
+	TC_Enable;
 	GC_Unlink;
+    }
+    form = Cdr (form);
+    t = TYPE(fun);
+    if (t == T_Macro) {
+	form = Macro_Expand (fun, form);
 	goto again;
     }
-    ret = Funcall (fun, args, 1);
-    GC_Unlink;
-    return ret;
+    Funcall_Switch (t, fun, form, 1);
+    /*NOTREACHED*/
 }
 
 Object P_Eval (argc, argv) Object *argv; {
@@ -158,137 +182,146 @@ Object P_Apply (argc, argv) Object *argv; {
 	if (Nullp (list))
 	    list = cell;
 	else
-	    (void)P_Setcdr (tail, cell);
+	    (void)P_Set_Cdr (tail, cell);
     }
     for (last = argv[argc-1]; !Nullp (last); last = Cdr (last), tail = cell) {
 	cell = Cons (P_Car (last), Null);
 	if (Nullp (list))
 	    list = cell;
 	else
-	    (void)P_Setcdr (tail, cell);
+	    (void)P_Set_Cdr (tail, cell);
     }
     ret = Funcall (argv[0], list, 0);
     GC_Unlink;
     return ret;
 }
 
-Arglist_Length (list) Object list; {
-    Object tail;
-    register i;
-
-    for (i = 0, tail = list; TYPE(tail) == T_Pair; tail = Cdr (tail), i++)
-	;
-    if (Nullp (tail))
-	return i;
-    Primitive_Error ("argument list is improper");
-    /*NOTREACHED*/
-}
-
 Object Funcall_Primitive (fun, argl, eval) Object fun, argl; {
     register struct S_Primitive *prim;
     register argc, i;
-    const char *last;
-    Object *argv;
-    Object abuf[MAX_ARGS_ON_STACK], ret;
-    GC_Node2; GCNODE gcv;
+    const char *last_tag;
+    register Object *argv;
+    Object abuf[MAX_ARGS_ON_STACK], r, e;
+    GC_Node4; GCNODE gcv;
     TC_Prolog;
     Alloca_Begin;
 
     prim = PRIM(fun);
-    last = Error_Tag;
+    last_tag = Error_Tag;   /* avoid function calls here */
     Error_Tag = prim->name;
-    argc = Arglist_Length (argl);
+    Get_Arglist_Length (argc, argl, r);  /* r is temporary variable */
     if (argc < prim->minargs
 	    || (prim->maxargs != MANY && argc > prim->maxargs))
 	Primitive_Error ("wrong number of arguments");
+
+    e = The_Environment;
+    GC_Link4_Tag_Primitive (argl, fun, e, r);
+
     if (prim->disc == NOEVAL) {
-	ret = (prim->fun)(argl);
+	r = (prim->fun)(argl);
     } else {
-	/* Tail recursion is not possible while evaluating the arguments
-	 * of a primitive procedure.
-	 */
 	TC_Disable;
-	if (argc <= MAX_ARGS_ON_STACK)
-	    argv = abuf;
-	else
-	    Alloca (argv, Object*, argc * sizeof (Object));
-	GC_Link2 (argl, fun);
-	gcv.gclen = 1; gcv.gcobj = argv; gcv.next = &gc2; GC_List = &gcv;
-	for (i = 0; i < argc; i++, argl = Cdr (argl)) {
-	    argv[i] = eval ? Eval (Car (argl)) : Car (argl);
-	    gcv.gclen++;
+	/*
+	 * Skip the loop if argc==0 or argc==1 (special case below).
+	 */
+	if (prim->disc != EVAL || argc >= 2) {
+	    if (argc <= MAX_ARGS_ON_STACK)
+		argv = abuf;
+	    else
+		Alloca (argv, Object*, argc * sizeof (Object));
+	    gcv.gclen = 1; gcv.gcobj = argv; gcv.next = &gc4; GC_List = &gcv;
+	    for (r = argl, i = 0; i < argc; i++, r = Cdr (r)) {
+		argv[i] = eval ? Eval (Car (r)) : Car (r);
+		gcv.gclen++;
+	    }
+	    TC_Enable;
+	    prim = PRIM(fun);   /* fun has possibly been moved during gc */
 	}
-	TC_Enable;
-	prim = PRIM(fun);   /* fun has possibly been moved during gc */
 	if (prim->disc == VARARGS) {
-	    ret = (prim->fun)(argc, argv);
+	    r = (prim->fun)(argc, argv);
 	} else {
 	    switch (argc) {
 	    case 0:
-		ret = (prim->fun)(); break;
+		r = (prim->fun)(); break;
 	    case 1:
-		ret = (prim->fun)(argv[0]); break;
+		TC_Disable;
+		r = eval ? Eval (Car (argl)) : Car (argl);
+		TC_Enable;
+		r = (PRIM(fun)->fun)(r);
+		break;
 	    case 2:
-		ret = (prim->fun)(argv[0], argv[1]); break;
+		r = (prim->fun)(argv[0], argv[1]); break;
 	    case 3:
-		ret = (prim->fun)(argv[0], argv[1], argv[2]); break;
+		r = (prim->fun)(argv[0], argv[1], argv[2]); break;
 	    case 4:
-		ret = (prim->fun)(argv[0], argv[1], argv[2], argv[3]); break;
+		r = (prim->fun)(argv[0], argv[1], argv[2], argv[3]); break;
 	    case 5:
-		ret = (prim->fun)(argv[0], argv[1], argv[2], argv[3], argv[4]);
+		r = (prim->fun)(argv[0], argv[1], argv[2], argv[3], argv[4]);
 		break;
 	    case 6:
-		ret = (prim->fun)(argv[0], argv[1], argv[2], argv[3], argv[4],
-		                  argv[5]); break;
+		r = (prim->fun)(argv[0], argv[1], argv[2], argv[3], argv[4],
+		                argv[5]); break;
 	    case 7:
-		ret = (prim->fun)(argv[0], argv[1], argv[2], argv[3], argv[4],
-		                  argv[5], argv[6]); break;
+		r = (prim->fun)(argv[0], argv[1], argv[2], argv[3], argv[4],
+		                argv[5], argv[6]); break;
 	    case 8:
-		ret = (prim->fun)(argv[0], argv[1], argv[2], argv[3], argv[4],
-		                  argv[5], argv[6], argv[7]); break;
+		r = (prim->fun)(argv[0], argv[1], argv[2], argv[3], argv[4],
+		                argv[5], argv[6], argv[7]); break;
 	    case 9:
-		ret = (prim->fun)(argv[0], argv[1], argv[2], argv[3], argv[4],
-		                  argv[5], argv[6], argv[7], argv[8]); break;
+		r = (prim->fun)(argv[0], argv[1], argv[2], argv[3], argv[4],
+		                argv[5], argv[6], argv[7], argv[8]); break;
 	    case 10:
-		ret = (prim->fun)(argv[0], argv[1], argv[2], argv[3], argv[4],
-		                  argv[5], argv[6], argv[7], argv[8], argv[9]);
+		r = (prim->fun)(argv[0], argv[1], argv[2], argv[3], argv[4],
+		                argv[5], argv[6], argv[7], argv[8], argv[9]);
 		break;
 	    default:
 		Panic ("too many args for primitive");
 	    }
 	}
-	GC_Unlink;
 	Alloca_End;
     }
-    Error_Tag = last;
-    return ret;
+    GC_Unlink;
+    Error_Tag = last_tag;
+    return r;
 }
 
-/* If we are in a tail recursion, we are reusing the old procedure
- * frame; we just assign new values to the formal parameters:
+/* This macro is used by Funcall_Compound() below.  Note that
+ * if we are in a tail recursion, we are reusing the old procedure
+ * frame; we just assign new values to the formal parameters.
+ * Add_Binding() has been inlined here for speed.  r is used as
+ * a temporary variable.
  */
-#define Lambda_Bind(var,val)\
-if (tail_calling) {\
-    newframe = Add_Binding (newframe, var, val);\
-} else {\
-    frame = Add_Binding (frame, var, val);\
+#define Lambda_Bind(var,val) {\
+    r = Cons (var, val);\
+    if (tail_calling)\
+	newframe = Cons (r, newframe);\
+    else\
+	frame = Cons (r, frame);\
 }
 
 Object Funcall_Compound (fun, argl, eval) Object fun, argl; {
     register argc, min, max, i, tail_calling = 0;
-    Object *argv, abuf[MAX_ARGS_ON_STACK], rest, ret, frame,
-	tail, tail_call_env, oldenv, newframe;
-    GC_Node6; GCNODE gcv;
-    TC_Prolog;
+    register Object *argv;
+    Object abuf[MAX_ARGS_ON_STACK], rest, r, frame, tail,
+	tail_call_env, oldenv, newframe;
+    register GCNODE *p;
+    GC_Node7; GCNODE gcv;
     Alloca_Begin;
 
-#ifdef lint
-    tail_call_env = Null;
-#endif
-    frame = oldenv = tail = newframe = Null;
-    GC_Link6 (argl, oldenv, frame, tail, fun, newframe);
+    if (Tail_Call && eval) {
+	for (p = GC_List; p && p->gclen != TAG_FUN; p = p->next) {
+	    if (p->gclen == TAG_TCFUN && EQ(*(p->gcobj), fun)) {
+		SET(r, T_Special, 0);
+		tc_fun = fun; tc_argl = argl; tc_env = The_Environment;
+		return r;
+	    }
+	}
+    }
+    r = frame = tail = newframe = Null;
+    oldenv = The_Environment;
+    GC_Link7_Tag_Compound (argl, fun, oldenv, frame, tail, newframe, r);
 again:
-    argc = Arglist_Length (argl);
+    Get_Arglist_Length (argc, argl, r);   /* r is temporary variable here */
     min = COMPOUND(fun)->min_args;
     max = COMPOUND(fun)->max_args;
     if (argc < min)
@@ -304,13 +337,12 @@ again:
 	else
 	    Alloca (argv, Object*, argc * sizeof (Object));
     }
-    TC_Disable;
-    gcv.gclen = 1; gcv.gcobj = argv; gcv.next = &gc6; GC_List = &gcv;
-    for (i = 0; i < argc; i++, argl = Cdr (argl)) {
-	argv[i] = eval ? Eval (Car (argl)) : Car (argl);
+    Tail_Call = 0;
+    gcv.gclen = 1; gcv.gcobj = argv; gcv.next = &gc7; GC_List = &gcv;
+    for (r = argl, i = 0; i < argc; i++, r = Cdr (r)) {
+	argv[i] = eval ? Eval (Car (r)) : Car (r);
 	gcv.gclen++;
     }
-    TC_Enable;
     if (tail_calling)
 	Switch_Environment (tail);
     tail = Car (Cdr (COMPOUND(fun)->closure));
@@ -324,20 +356,21 @@ again:
 	Pop_Frame ();
 	Push_Frame (newframe);
     } else {
-	oldenv = The_Environment;
 	Switch_Environment (COMPOUND(fun)->env);
         Push_Frame (frame);
     }
-
+    tail = Cdr (Cdr (COMPOUND(fun)->closure));
+    for (i = COMPOUND(fun)->numforms; i > 1; i--, tail = Cdr (tail))
+	(void)Eval (Car (tail));
     Tail_Call = 1;
-    ret = Begin (Cdr (Cdr (COMPOUND(fun)->closure)));
+    r = Eval (Car (tail));
     /*
      * If evaluation of the function body returned a T_Special object,
      * a tail-call has been taken place.  If it is a tail-call to a
      * different function, just return, otherwise unpack new arguments
      * and environment and jump to the beginning.
      */
-    if (TYPE(ret) == T_Special && EQ(fun, tc_fun)) {
+    if (TYPE(r) == T_Special && EQ(fun, tc_fun)) {
 	argl = tc_argl;
 	tail_call_env = tc_env;
 	tail_calling = 1;
@@ -350,45 +383,13 @@ again:
     Switch_Environment (oldenv);
     GC_Unlink;
     Alloca_End;
-    return ret;
+    return r;
 }
 
 Object Funcall (fun, argl, eval) Object fun, argl; {
-    register t;
-    register GCNODE *p;
-    Object ret, env;
-    Tag_Node;
-
-    t = TYPE(fun);
-    /* Search upwards in the GC list for a TAG frame pointing to
-     * the function we are abount to call.  Stop if a TAG frame
-     * is encountered that points to a function call that is not
-     * in a tail-call position.
-     *
-     * If the search succeeds, package up function, actual arguments,
-     * and environment, and return a T_Special object.
-     */
-    if (Tail_Call && eval && t == T_Compound) {
-	for (p = GC_List; p && p->gclen != TAG_FUN; p = p->next) {
-	    if (p->gclen == TAG_TCFUN && EQ(*(p->gcobj), fun)) {
-		SET(ret, T_Special, 0);
-		tc_fun = fun; tc_argl = argl; tc_env = The_Environment;
-		return ret;
-	    }
-	}
-    }
-    env = The_Environment;
-    Tag_Link (argl, fun, env);
-    if (t == T_Primitive) {
-	ret = Funcall_Primitive (fun, argl, eval);
-    } else if (t == T_Compound) {
-	ret = Funcall_Compound (fun, argl, eval);
-    } else if (t == T_Control_Point) {
-	Funcall_Control_Point (fun, argl, eval);
-	/*NOTREACHED*/
-    } else Primitive_Error ("application of non-procedure: ~s", fun);
-    Tag_Unlink;
-    return ret;
+    register t = TYPE(fun);
+    Funcall_Switch (t, fun, argl, eval);
+    /*NOTREACHED*/
 }
 
 Check_Formals (x, min, max) Object x; int *min, *max; {
@@ -398,7 +399,7 @@ Check_Formals (x, min, max) Object x; int *min, *max; {
     for (t1 = Car (x); !Nullp (t1); t1 = Cdr (t1)) {
         s = TYPE(t1) == T_Pair ? Car (t1) : t1;
 	Check_Type (s, T_Symbol);
-	for (t2 = Car (x); t2 != t1; t2 = Cdr (t2))
+	for (t2 = Car (x); !EQ(t2, t1); t2 = Cdr (t2))
 	    if (EQ(s, Car (t2)))
 		Primitive_Error ("~s: duplicate variable binding", s);
 	if (TYPE(t1) != T_Pair)
@@ -421,6 +422,7 @@ Object P_Lambda (argl) Object argl; {
     closure = Cons (Sym_Lambda, argl);
     COMPOUND(proc)->closure = closure;
     COMPOUND(proc)->env = The_Environment;
+    COMPOUND(proc)->numforms = Fast_Length (Cdr (argl));
     Check_Formals (argl, &COMPOUND(proc)->min_args,
 	&COMPOUND(proc)->max_args);
     GC_Unlink;
@@ -432,7 +434,7 @@ Object P_Procedure_Lambda (p) Object p; {
     return Copy_List (COMPOUND(p)->closure);
 }
 
-Object P_Procedure_Env (p) Object p; {
+Object P_Procedure_Environment (p) Object p; {
     Check_Type (p, T_Compound);
     return COMPOUND(p)->env;
 }
@@ -472,7 +474,7 @@ Object General_Map (argc, argv, accum) Object *argv; register accum; {
 	if (Nullp (list))
 	    list = cell;
 	else
-	    (void)P_Setcdr (tail, cell);
+	    (void)P_Set_Cdr (tail, cell);
 	tail = cell;
     }
     /*NOTREACHED*/
@@ -514,15 +516,14 @@ Object P_Macro_Body (m) Object m; {
 }
 
 Object Macro_Expand (mac, argl) Object mac, argl; {
-    register argc, min, max, i, tail_calling = 0;
-    Object frame, ret, tail;
-    Object newframe; /* not used; see Lambda_Bind() */
+    register argc, min, max, i;
+    Object frame, r, tail;
     GC_Node4;
     TC_Prolog;
 
     frame = tail = Null;
     GC_Link4 (argl, frame, tail, mac);
-    argc = Arglist_Length (argl);
+    Get_Arglist_Length (argc, argl, r);
     min = MACRO(mac)->min_args;
     max = MACRO(mac)->max_args;
     if (argc < min)
@@ -531,16 +532,16 @@ Object Macro_Expand (mac, argl) Object mac, argl; {
 	Primitive_Error ("too many arguments for ~s", mac);
     tail = Car (Cdr (MACRO(mac)->body));
     for (i = 0; i < min; i++, tail = Cdr (tail), argl = Cdr (argl))
-	Lambda_Bind (Car (tail), Car (argl));
+	frame = Add_Binding (frame, Car (tail), Car (argl));
     if (max == -1)
-	Lambda_Bind (tail, argl);
+	frame = Add_Binding (frame, tail, argl);
     Push_Frame (frame);
     TC_Disable;
-    ret = Begin (Cdr (Cdr (MACRO(mac)->body)));
+    r = Begin (Cdr (Cdr (MACRO(mac)->body)));
     TC_Enable;
     Pop_Frame ();
     GC_Unlink;
-    return ret;
+    return r;
 }
 
 Object P_Macro_Expand (form) Object form; {
